--
-- EquipCompare by Legorol
-- Version: 2.0.2
--
-- Equipment comparison tooltips, not just at the merchant. If you hover over
-- any item in places like your bag, quest reward page or loot window, you
-- get a comparison tooltip showing the item of that type that you currently
-- have equipped.
--
-- Usage:
-- To get usage information, type /equipcompare help in-game.
--

--
-- Global variables
--

EquipCompare_Enabled = true;
EquipCompare_Recheck = true;
EquipCompare_Protected = false;
EquipCompare_Showing = false;
EquipCompare_CharactersViewer = false;

EquipCompare_ItemTypes = {
	[INVTYPE_WEAPONMAINHAND] = 16, -- Main Hand
	[INVTYPE_2HWEAPON] = 16, -- Two-Hand
	[INVTYPE_WEAPON] = 16, -- One-Hand
	[INVTYPE_WEAPON.."_other"] = 17, -- One-Hand_other
--  Obsolete entries?
--	["Hand"] = 16,
--	["Held In Hand"] = 16,
	[INVTYPE_SHIELD] = 17, -- Off Hand
	[INVTYPE_WEAPONOFFHAND] = 17, -- Off Hand
	[INVTYPE_HOLDABLE] = 17, -- Held In Off-hand
	[INVTYPE_HEAD] = 1, -- Head
	[INVTYPE_WAIST] = 6, -- Waist
	[INVTYPE_SHOULDER] = 3, -- Shoulder
	[INVTYPE_LEGS] = 7, -- Legs
	[INVTYPE_CLOAK] = 15, -- Back
	[INVTYPE_FEET] = 8, -- Feet
	[INVTYPE_CHEST] = 5, -- Chest
	[INVTYPE_ROBE] = 5, -- Chest
	[INVTYPE_WRIST] = 9, -- Wrist
	[INVTYPE_HAND] = 10, -- Hands
	[INVTYPE_RANGED] = 18, -- Ranged
	[INVTYPE_BODY] = 4, -- Shirt
	[INVTYPE_TABARD] = 19, -- Tabard
	[INVTYPE_FINGER] = 11, -- Finger
	[INVTYPE_FINGER.."_other"] = 12, -- Finger_other
	[INVTYPE_NECK] = 2, -- Neck
	[INVTYPE_TRINKET] = 13, -- Trinket
	[INVTYPE_TRINKET.."_other"] = 14, -- Trinket_other
	[INVTYPE_WAND] = 18, -- Wand
	[INVTYPE_GUN] = 18, -- Gun
	[INVTYPE_GUNPROJECTILE] = 0, -- Projectile
	[INVTYPE_BOWPROJECTILE] = 0 -- Projectile
};

--
-- XML Event handlers
--

function EquipCompare_OnLoad()
	this:RegisterEvent("CLEAR_TOOLTIP");
	-- Check for Cosmos. If available, register with it.
	if ( Cosmos_RegisterConfiguration ) then
		this:RegisterEvent("VARIABLES_LOADED");
		Cosmos_RegisterConfiguration(
			"COS_EQC",
			"SECTION",
			EQUIPCOMPARE_COSMOS_SECTION,
			EQUIPCOMPARE_COSMOS_SECTION_INFO
		);
		Cosmos_RegisterConfiguration(
			"COS_EQC_SEPARATOR",
			"SEPARATOR",
			EQUIPCOMPARE_COSMOS_HEADER,
			EQUIPCOMPARE_COSMOS_HEADER_INFO
			
		);
		Cosmos_RegisterConfiguration(
			"COS_EQC_ENABLED",
			"CHECKBOX",
			EQUIPCOMPARE_COSMOS_ENABLE,
			EQUIPCOMPARE_COSMOS_ENABLE_INFO,
			EquipCompare_Toggle,
			1
		);
	end
	-- If Cosmos allows chat command registration, do so
	if ( Cosmos_RegisterChatCommand ) then
		local comlist = { "/equipcompare", "/eqc" };
		local desc = EQUIPCOMPARE_COSMOS_SLASH_DESC;
		local id = "EQUIPCOMPARE";
		local func = EquipCompare_SlashCommand
		Cosmos_RegisterChatCommand ( id, comlist, func, desc, CSM_CHAINNONE );
	else
	-- otherwise, just register slash commands manually
		SlashCmdList["EQUIPCOMPARE"] = EquipCompare_SlashCommand;
		SLASH_EQUIPCOMPARE1 = "/equipcompare";
		SLASH_EQUIPCOMPARE2 = "/eqc";
	end
	
	-- Check to see if CharactersViewer is installed, has the right version
	-- and has the required interface. If so, enable support for it.
	if ( CHARACTERSVIEWER_VERSION and CHARACTERSVIEWER_VERSION >= 29 and
		type(CharactersViewer_Tooltip_SetInventoryItem)=="function") then
		EquipCompare_CharactersViewer = true;
	end
end

function EquipCompare_OnEvent(event)
	if ( event == "CLEAR_TOOLTIP" ) then
		if ( not EquipCompare_Protected ) then
			EquipCompare_Recheck = true;
			EquipCompare_HideTips();
		end
	elseif ( event == "VARIABLES_LOADED" ) then
		-- we have Cosmos, so let's read the state of the AddOn from Cosmos
		EquipCompare_Toggle(COS_EQC_ENABLED_X);
	end
end

function EquipCompare_OnUpdate()
	EquipCompare_ShowCompare();
end

--
-- Other functions
--

function EquipCompare_SlashCommand(msg)
	if (not msg or msg == "") then
		-- toggle
		EquipCompare_Toggle(not EquipCompare_Enabled);
	elseif (msg == "on") then
		-- turn on
		EquipCompare_Toggle(true);
	elseif (msg == "off") then
		-- turn off
		EquipCompare_Toggle(false);
	else
		-- usage
		ChatFrame1:AddMessage(EQUIPCOMPARE_USAGE_TEXT);
	end
	-- update Cosmos configuration setting
	if ( Cosmos_RegisterConfiguration ) then
		local newvalue;
		if ( EquipCompare_Enabled ) then
			newvalue = 1;
		else
			newvalue = 0;
		end
		COS_EQC_ENABLED_X=newvalue;
		Cosmos_UpdateValue("COS_EQC_ENABLED", CSM_CHECKONOFF, newvalue);
		Cosmos_SetCVar("COS_EQC_ENABLED_X", newvalue);
	end
end

function EquipCompare_Toggle(toggle)
	-- workaround, as Cosmos sends a 0 when it wants to turn us off
	if ( toggle == 0 ) then toggle = false; end
	-- turn on
	if ( toggle and not EquipCompare_Enabled ) then
		EquipCompare_Enabled = true;
		EquipCompare_Recheck = true;
	end
	-- turn off
	if ( not toggle and EquipCompare_Enabled ) then
		EquipCompare_Enabled = false;
		EquipCompare_HideTips();
	end
end

function EquipCompare_HideTips()
	if ( EquipCompare_Showing ) then
		EquipCompare_Showing = false;
		EquipCompare_CarefulHideTooltip(ShoppingTooltip1);
		EquipCompare_CarefulHideTooltip(ShoppingTooltip2);
	end
end

-- The following function is responsible for hiding a tooltip without
-- triggering a call to whatever the current GameTooltip_OnHide function is,
-- merely performing those steps instead that the default Blizzard UI implementation
-- of GameTooltip_OnHide does.
-- This is necessary because many AddOn authors mistakenly assume that
-- GameTooltip_OnHide gets called *only* when it is the GameTooltip that
-- gets hidden, and override GameTooltip_OnHide accordingly. Unfortunately
-- GameTooltip_OnHide gets called when *any* tooltip gets hidden, so hiding
-- the ShoppingTooltips can have unintended side effects in other AddOns.
-- This function is therefore here only as a workaround.
function EquipCompare_CarefulHideTooltip(tooltip)
	local temp_GameTooltip_OnHide;
	if ( tooltip and tooltip:IsVisible() ) then
		temp_GameTooltip_OnHide = GameTooltip_OnHide;
		GameTooltip_OnHide = function () end
		tooltip:Hide();
		tooltip:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b);
		tooltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b);
		tooltip.default = nil;
		GameTooltip_OnHide = temp_GameTooltip_OnHide;
	end
end

-- This function is called on every OnUpdate. This is the only way to detect if a
-- game tooltip has been displayed, without overriding FrameXML files.
-- So that this function doesn't hog resources, the EquipCompare_Recheck flag is
-- set to false, until the game tooltip changes.
function EquipCompare_ShowCompare()
	--
	-- Local functions
	--
	
	-- Display a shopping tooltip and set its contents to currently equipped item
	-- occupying slotid
	local function ShowShoppingTooltip(parent, slotid)
		local leftAlign=true;
		ShoppingTooltip1:SetOwner(getglobal(parent), "ANCHOR_NONE");
		ShoppingTooltip1:SetPoint("TOPRIGHT", parent, "TOPLEFT", 0, -10);
		if ( EquipCompare_CharactersViewer ) then
			CharactersViewer_Tooltip_SetInventoryItem(ShoppingTooltip1, slotid);
		else
			ShoppingTooltip1:SetInventoryItem("player", slotid);
		end
		local left = ShoppingTooltip1:GetLeft();
		-- If the shopping tooltip would be off the screen, place it on other 
		-- side instead. Also, if we are over a QuestFrame, or the tooltip
		-- is a LootLink tooltip or ItemRef tooltip, place it on right
		if ( (left and left<0) or 
			  (QuestFrame:IsVisible() and MouseIsOver(QuestFrame)) or
			  parent == "ItemRefTooltip" or
			  parent == "LootLinkTooltip" ) then
			ShoppingTooltip1:ClearAllPoints();
			ShoppingTooltip1:SetPoint("TOPLEFT", parent, "TOPRIGHT", 0, -10);
			leftAlign=false;
		end
		return leftAlign
	end
	
	--
	-- Main code of EquipCompare_ShowCompare starts here
	--
	local OverrideTooltips = nil;
	local tooltip, ttext, itype, slotid, other, leftAlign;
	local i,cvplayer;
	
	if ( not EquipCompare_Enabled ) then
		return;
	end

	if ( (not GameTooltip or not GameTooltip:IsVisible()) and
		  (not LootLinkTooltip or not LootLinkTooltip:IsVisible()) and
		  (not ItemRefTooltip or not ItemRefTooltip:IsVisible()) ) then
		EquipCompare_Recheck = true;
		EquipCompare_HideTips();
		return;
	end
	
	if ( not EquipCompare_Recheck ) then
	 	return;
	end
	
	-- Check if it's the GameTooltip or the LootLinkTooltip we are interested in
	if ( ItemRefTooltip and ItemRefTooltip:IsVisible() ) then
		tooltip = "ItemRefTooltip";
	elseif ( LootLinkTooltip and LootLinkTooltip:IsVisible() ) then
		tooltip = "LootLinkTooltip";
	elseif ( GameTooltip and GameTooltip:IsVisible() ) then
		tooltip = "GameTooltip";
	else
		-- We should never reach this point. If we do, it's a disaster, bail out.
		return;
	end

	EquipCompare_Recheck = false;
	
	-- In some cases it is desirable to override the comparison tooltips already
	-- provided by e.g. Merchant and Auction House. Check for that here.
	if ( EquipCompare_CharactersViewer ) then
		if ( CharactersViewer_Frame:IsVisible() and
			MouseIsOver(CharactersViewer_Frame) ) then
			return;
		end
		cvplayer = CharactersViewerGetBSIIndex();
		if ( UnitName("player") ~= cvplayer ) then
			OverrideTooltips = true;
		end
	end
	
	-- Some frames provide equipment comparison tooltips already, so we don't need to,
	-- which we detect via ShoppingTooltip1 already being visible. Also, we don't need
	-- comparison tooltips when over e.g. PaperDollFrame. This is all true unless there
	-- is some special-case reason to override the tooltips.
	if ( OverrideTooltips ) then
		EquipCompare_HideTips();
	else
		if ( ( PaperDollFrame:IsVisible() and MouseIsOver(PaperDollFrame) ) or
			 ( ShoppingTooltip1 and ShoppingTooltip1:IsVisible() ) ) then
			return;
		end
	end
	
	-- Infer the type of the item from one of the 2nd to 5th line of its tooltip description
	-- Match this type against the appropriate slot
	slotid = nil;
	i = 2;
	repeat
		ttext = getglobal(tooltip.."TextLeft"..i);
		if ( ttext and ttext:IsVisible() ) then
			itype = ttext:GetText();
			if ( itype ) then
				slotid = EquipCompare_ItemTypes[itype];
			end
		end
		i = i + 1;
	until (slotid or i > 5)
	
	if ( slotid ) then
		-- Whilst we are in the process of displaying additional tooltips, we don't
		-- want to reset the EquipCompare_Recheck flag. This protection is necessary
		-- because calling SetOwner or SetxxxItem on any tooltip causes a
		-- CLEAR_TOOLTIP event.
	    EquipCompare_Protected = true;
		EquipCompare_Showing = true;
		
		-- In case money line is visible on GameTooltip, must protect it by overriding
		-- GameTooltip_ClearMoney. This is because calling SetOwner or SetxxxItem on
		-- any tooltip causes money line of GameTooltip to be cleared.
		local oldFunction = GameTooltip_ClearMoney;
		GameTooltip_ClearMoney = function() end;
		
		-- Display a shopping tooltip and set its contents to currently equipped item
		leftAlign = ShowShoppingTooltip(tooltip,slotid);
		
		other = false;
		-- If this is an item that can go into multiple slots, display additional
		-- tooltips as appropriate
		if ( itype == INVTYPE_FINGER ) then
			other = EquipCompare_ItemTypes[INVTYPE_FINGER.."_other"];
		end
		if ( itype == INVTYPE_TRINKET ) then
			other = EquipCompare_ItemTypes[INVTYPE_TRINKET.."_other"];
		end
		if ( itype == INVTYPE_WEAPON ) then
			other = EquipCompare_ItemTypes[INVTYPE_WEAPON.."_other"];
		end
		
		if ( other ) then
			if ( ShoppingTooltip1:IsVisible() ) then
				ShoppingTooltip2:SetOwner(ShoppingTooltip1, "ANCHOR_NONE");
				if ( leftAlign ) then
					ShoppingTooltip2:SetPoint("TOPRIGHT", "ShoppingTooltip1", "TOPLEFT", 0, 0);
				else
					ShoppingTooltip2:SetPoint("TOPLEFT", "ShoppingTooltip1", "TOPRIGHT", 0, 0);
				end
				if ( EquipCompare_CharactersViewer ) then
					CharactersViewer_Tooltip_SetInventoryItem(ShoppingTooltip2, other);
				else
					ShoppingTooltip2:SetInventoryItem("player", other);
				end
			else
				ShowShoppingTooltip(tooltip,other);
			end
		end
		
		-- Restore GameTooltip_ClearMoney overriding.
		GameTooltip_ClearMoney = oldFunction;
		
		EquipCompare_Protected = false;
	end
end
