--[[

	Loc: a simple in-game Location window
		copyright 2005 by Zavier
		Version 1.4


]]

--------------------------------------------------------------------------------------------------
-- Localizable strings
--------------------------------------------------------------------------------------------------

LOC = "Location";

LOC_RESET = "reset";
LOC_RESET_DONE = "Location: position reset to default";

LOC_REFRESH = "refresh";
LOC_REFRESH_DONE = "Location: Location refreshed";

LOC_SHOW = "show";
LOC_SHOW_DONE = "Location: Button shown";

LOC_HIDE = "hide";
LOC_HIDE_DONE = "Location: Button hidden type /loc show to show it again";

LOC_HELP = "help";

LOC_HELP_TEXT0 = " ";
LOC_HELP_TEXT1 = "|cffffff00Loc command help:|r";
LOC_HELP_TEXT2 = "|cff00ff00Use |r|cffffffff/loc <command>|r|cff00ff00 to perform the following commands:|r";
LOC_HELP_TEXT3 = "|cffffffff"..LOC_HELP.."|r|cff00ff00: displays this message.|r";
LOC_HELP_TEXT4 = "|cffffffff"..LOC_SHOW.."|r|cff00ff00: displays the location button.|r";
LOC_HELP_TEXT5 = "|cffffffff"..LOC_HIDE.."|r|cff00ff00: hides the location button.|r";
LOC_HELP_TEXT6 = "|cffffffff"..LOC_RESET.."|r|cff00ff00: resets the location button to its default position.|r";
LOC_HELP_TEXT7 = "|cffffffff"..LOC_REFRESH.."|r|cff00ff00: refreshes your location in case it gets set to 0,0 if you are outside an instance.|r";
LOC_HELP_TEXT8 = "|cffffffff".."/loc".."|r|cff00ff00 with nothing after it displays your location in the chat window.|r";

LOC_HELP_TEXT = {
	LOC_HELP_TEXT0,
	LOC_HELP_TEXT1,
	LOC_HELP_TEXT2,
	LOC_HELP_TEXT3,
	LOC_HELP_TEXT4,
	LOC_HELP_TEXT5,
	LOC_HELP_TEXT6,
	LOC_HELP_TEXT7,
	LOC_HELP_TEXT8,
};



LocZoneData = { };



--------------------------------------------------------------------------------------------------
-- Local variables
--------------------------------------------------------------------------------------------------

local lBeingDragged;

--------------------------------------------------------------------------------------------------
-- Global variables
--------------------------------------------------------------------------------------------------

-- Constants
LOC_UPDATE_RATE = 0.1;

--------------------------------------------------------------------------------------------------
-- Internal functions
--------------------------------------------------------------------------------------------------

local function Loc_Reset()
	LocFrame:ClearAllPoints();
	LocFrame:SetPoint("TOP", "UIParent", "TOP", 128, 0);
end

function Loc_SlashCommandHandler(msg)
	if( msg ) then
		local command = string.lower(msg);
		if( command == "" ) then
			local index;
			local value;
			value="Location Coordinates are: "..Loc_PlayerPos();
			DEFAULT_CHAT_FRAME:AddMessage(value);
		elseif( command == LOC_RESET ) then
			Loc_Reset();
			Loc_OnDragStop();
			DEFAULT_CHAT_FRAME:AddMessage(LOC_RESET_DONE);
		elseif( command == LOC_SHOW) then
			LocFrame:Show();
			DEFAULT_CHAT_FRAME:AddMessage(LOC_SHOW_DONE);
		elseif(command == LOC_HIDE) then
			LocFrame:Hide();
			DEFAULT_CHAT_FRAME:AddMessage(LOC_HIDE_DONE);
		elseif( command == LOC_REFRESH ) then
			local px, py = Loc_PlayerPos();
			if ((px == 0) and (py == 0)) then 
				DEFAULT_CHAT_FRAME:AddMessage("Player is not on a visible map");
			end
			DEFAULT_CHAT_FRAME:AddMessage(LOC_REFRESH_DONE);
		elseif( command == LOC_HELP) then
			for index, value in LOC_HELP_TEXT do
				DEFAULT_CHAT_FRAME:AddMessage(value);
			end
		end
	end
end

--------------------------------------------------------------------------------------------------
-- Functions
--------------------------------------------------------------------------------------------------

function Loc_OnLoad()
	this:RegisterForDrag("LeftButton");

	Loc_LoadContinents(GetMapContinents());
	
	-- RegisterForSave("LocState");
	-- Register our slash command
	SLASH_LOC1 = "/loc";
	SlashCmdList["LOC"] = function(msg)
		Loc_SlashCommandHandler(msg);
	end
	
	LocFrame.TimeSinceLastUpdate = 0;

	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("Zavier's Location AddOn loaded");
	end
	UIErrorsFrame:AddMessage("Zavier's Location AddOn loaded v1.5", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
end

function Loc_OnUpdate(arg1)

	LocFrame.TimeSinceLastUpdate = LocFrame.TimeSinceLastUpdate + arg1;
	if( LocFrame.TimeSinceLastUpdate > LOC_UPDATE_RATE ) then
		LocText:SetText(Loc_PlayerPos());
		LocFrame.TimeSinceLastUpdate = 0;
	end

end

function Loc_OnDragStart()
	LocFrame:StartMoving()
	lBeingDragged = 1;
end

function Loc_OnDragStop()
	LocFrame:StopMovingOrSizing()
	lBeingDragged = nil;
end

function Loc_LoadContinents(...)
	LocZoneData = { };
	for c=1, arg.n, 1 do
		local cname = arg[c];
		Loc_LoadZones(c, cname, GetMapZones(c));
	end
end

function Loc_LoadZones(c,cname, ...)
	for z=1, arg.n, 1 do
		local zname = arg[z];
		LocZoneData[zname] = { continent=c, zone=z, continentName=cname };
	end
end

function Loc_GetCurrentZone()
	local currentZoneName = GetZoneText();
	for zoneName, zoneData in LocZoneData do
		if (currentZoneName == zoneName) then
			return zoneData.continent, zoneData.zone;
		end
	end
	return 0,0;
end
	
function Loc_ChangeMap()
	local mapContinent = GetCurrentMapContinent();
	local mapZone = GetCurrentMapZone();
	local playerContinent, playerZone = Loc_GetCurrentZone();
	if ((playerContinent == 0) or (playerZone == 0)) then return false; end
	if ((playerContinent == mapContinent) and (playerZone == mapZone)) then return true; end

	SetMapZoom(playerContinent, playerZone);
	return true;
end

function Loc_PlayerPos(forced)
	local out;
	local currentZoneName = GetZoneText();
	if (currentZoneName ~= Gather_LastZone) then
		forced = true;
	end
	Loc_LastZone = currentZoneName;
	if (forced) then Loc_ChangeMap(); end

	local px, py = GetPlayerMapPosition("player");
	if ((px == 0) and (py == 0)) then
		if (not forced) then 
			if (Loc_ChangeMap()) then
				px, py = GetPlayerMapPosition("player");
			else
				return 0,0;
			end
		end
	end

	out="X:" .. floor(px*100) .. " Y:" .. floor(py*100); 

	return format(out);

end

