--[[

MapNotesNoteTarget 
        Author:         chrispix (mainly copied from Sir.Bender's MapNotes code)
        Version:        0.5.3: Initial release
        ToDo:           Send requests for functionality to mapnotes@cricklepickle.com.
        Last Modified   
		1/31/2005 	Release 0.5.3
                    Initial release.
					
		An extension to the excellent MapNotes AddOn which allows you to quickly note your current
        position, as well as some information on your current target. Useful for remembering the 
        location of questgivers, trainers, flight masters, named mobs, types of critters, etc.

        If the target is friendly, the icon will be green and contain the target's name.
        If the target is neutral (usually critters), the icon will be yellow and contain the target's name.
        If the target is hostile, the icon will be red and contain the target's name, level, elite/boss status,
        type (beast, humanoid, elemental, etc), and class (warrior, rogue, warlock, etc).

        If you type any text after the command, it will be added to the first optional text line in the note. 
        For example, /nt Flight master, /nt Alchemy trainer, /nt has bodyguards, etc.

        By default, the note will be created on the world map. However, if you'd like to create a minimap note,
        it works in the same way as MapNotes. Just type /nmn to specify your next note as a minimap note, then
        create the note.

        Usage: Target something and type /nt [optional text] or /notetarget [optional text]

        Requires MapNotes version 0.5.3 or later, available at http://www.wowwiki.com/Map_Notes. 
        It *may* be compatible with previous versions, but I make no guarantees. If you have any problems 
        with a version later than 0.5.3, email me at mapnotes@cricklepickle.com and I'll look into it.
]]

function MapNotesNoteTarget_OnLoad()
	if ( ( Cosmos_RegisterConfiguration ) and ( MapNotes_CosmosIsRegistered == false ) ) then
		if ( Cosmos_RegisterChatCommand ) then
		local MapNotesNoteTargetCommands = {"/nt", "/notetarget"};
		Cosmos_RegisterChatCommand (
			"MAPNOTES_NT_COMMANDS",
			MapNotesNoteTargetCommands,
			MapNotes_NoteTarget,
			MAPNOTES_CHAT_COMMAND_NT
			);
		end
        --MapNotes_StatusPrint("Loaded MapNotesNoteTarget with Cosmos");
	else
		SLASH_NT1 = "/nt";
		SLASH_NT2 = "/notetarget";
		SlashCmdList["NT"] = MapNotesNoteTarget_CreateNote;
        --MapNotes_StatusPrint("Loaded MapNotesNoteTarget without Cosmos");
	end
end

function MapNotesNoteTarget_CreateNote(msg)
	local continent, zone = MapNotes_GetZone();
	SetMapZoom(continent, zone);
	local x, y = GetPlayerMapPosition("player");
	local checknote = MapNotes_CheckNearNotes(continent, zone, x, y);
	if (checknote) then
		MapNotes_StatusPrint(format(MAPNOTES_QUICKNOTE_NOTETONEAR, MapNotes_Data[continent][zone][checknote].name));
	elseif ((x == 0 and y == 0) or zone == 0) then
		MapNotes_StatusPrint(MAPNOTES_QUICKNOTE_NOPOSITION);
	else
		local id = 0;
		local icon = 3; --green by default

        local text = UnitName("target");
        if (UnitReaction("player", "target") < 4) then 
            -- hostile, get level, classification, type, and class
            text = text.." level "..UnitLevel("target");
            icon = 1; --red
            if (UnitClassification("target") ~= "normal") then
                text = text.." "..UnitClassification("target");
            end
            text = text.." "..UnitCreatureType("target").." "..UnitClass("target");
        elseif (UnitReaction("player", "target") == 4) then 
            -- neutral, assume critter, use yellow icon
            icon = 0; --yellow
        end

        --MapNotes_StatusPrint(text);
		if (MapNotes_SetNextAsMiniNote ~= 2) then
			local i = 0;
			for j, value in MapNotes_Data[continent][zone] do
				i = i + 1;
			end
			if (i < MapNotes_NotesPerZone) then
				MapNotes_TempData_Id = i + 1;
				MapNotes_Data[continent][zone][MapNotes_TempData_Id] = {};
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].name = text;
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].ncol = 0; -- name color
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].inf1 = msg; -- info 1 text
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].in1c = 0; -- info 1 color
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].inf2 = "";
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].in2c = 0;
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].creator = UnitName("player");
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].icon = icon; -- icon color
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].xPos = x;
				MapNotes_Data[continent][zone][MapNotes_TempData_Id].yPos = y;
				id = MapNotes_TempData_Id;
				MapNotes_StatusPrint(format(MAPNOTES_QUICKNOTE_OK, MapNotes_ZoneNames[continent][zone]).." for "..UnitName("target"));
			else
				MapNotes_StatusPrint(format(MAPNOTES_QUICKNOTE_TOOMANY, MapNotes_ZoneNames[continent][zone]));
			end
		end
		if (MapNotes_SetNextAsMiniNote ~= 0) then
			MapNotes_MiniNote_Data.xPos = x;
			MapNotes_MiniNote_Data.yPos = y;
			MapNotes_MiniNote_Data.continent = continent;
			MapNotes_MiniNote_Data.zone = zone;
			MapNotes_MiniNote_Data.id = id; -- only shown if the note was written...
			MapNotes_MiniNote_Data.name = text;
			MapNotes_MiniNote_Data.color = 0;
			MapNotes_MiniNote_Data.icon = icon;
			MiniNotePOITexture:SetTexture("Interface\\AddOns\\MapNotes\\POIIcons\\Icon"..icon);
			MiniNotePOI:Show();
			MapNotes_SetNextAsMiniNote = 0;
			MapNotes_StatusPrint(MAPNOTES_SETMININOTE.." for "..UnitName("target"));
		end
	end	
end

