--[[

ShardTracker - A Warlock AddOn for World of Warcraft 

REQUIRES: SEA - A general lua function library for WoW.
(Works with, but doesn't require, Cosmos)

Concept by Kithador 
Originally written by Kithador (ShartCount)
Sorting code originally by Ryu (ShartSort)
Modified, rewritten by Cragganmore 

This mod helps Warlocks to: 

o track how many soul shards you have 
o sort your soul shards into a specified bag 
o keep track of your healthstone status 
o create and use healthstones 
o give healthstones to other players 
o track your party members' healthstone status 
and be notified when they need new healthstones 
o keep track of your soulstone status 
o create and use soulstones 
o see the cooldown time until you can re-cast soulstone, 
and be notified when the cooldown time expires 
o keep track of your spellstone status 
o create and equip spellstones 
o see the cooldown time until you can use an equipped spellstone 
o automatically re-equip the item that was in your off hand when you 
equipped the spellstone 

For non-warlocks, this AddOn will display a small button 
near the minimap when the player has a healthstone. Clicking on this 
button will activate the healthstone. 

Installation: 

Unzip ShardTracker.zip into your WoW folder. Answer YES to the 
prompt about replacing any duplicate files. 

------- IMPORTANT! ------- IMPORTANT! ------- IMPORTANT! ------- 

NOTE: For this AddOn to create the little Healthstone icons above
your party members' profiles that notify you of their Healthstone
status, it MUST modify the PartyFrame.xml file. You have three options: 

If you follow the above installation instructions, the included 
PartyFrame.xml will unzip into into your Interface/FrameXML 
directory and you shouldn't have to do anything else. 

Alternately, you may modify the PartyFrame.xml file by hand 
with the XML code found at the end of the ShardTracker.lua 
file. This is the code that creates the buttons near your party 
members' portraits that let you know their Healthstone 
status. SEE END OF THIS FILE FOR THE XML CODE. 

You may also just choose to disregard this entirely, and the 
party Healthstone icons simply won't appear.  All other aspects 
of the plugin will still work fine.

For screenshots and downloads: 
http://www.curse-gaming.com/mod.php?addid=324 

]]--

SHARDTRACKER_DEBUG = false;
SHARDTRACKER_TRON  = false;
SHARDTRACKER_VERSION = "1.1";

SHARDTRACKER_FLASH_TIME_ON          = 0.75;     -- number of seconds to flash on
SHARDTRACKER_FLASH_TIME_OFF         = 0.75;     -- number of seconds to flash off
SHARDTRACKER_MIN_ALPHA              = 0.3;      -- minimum transparency to drop to when flashing
SHARDTRACKER_FIRST_TIME = { };                  -- used to track when certain events happen for the first time
SHARDTRACKER_FIRST_TIME.ENABLED         = true;
SHARDTRACKER_FIRST_TIME.SOUND           = true;
SHARDTRACKER_FIRST_TIME.MONITOR         = true;
SHARDTRACKER_FIRST_TIME.SOULPOP         = true;
SHARDTRACKER_FIRST_TIME.SHARDBG         = true;
SHARDTRACKER_FIRST_TIME.LIST_RESTRICT   = true;

-- default locations of the various buttons on the interface
SHARDTRACKER_ORIG_SHARDBUTTONX      = 42;
SHARDTRACKER_ORIG_SHARDBUTTONY      = -100;
SHARDTRACKER_ORIG_HEALTHBUTTONX     = 70;              
SHARDTRACKER_ORIG_HEALTHBUTTONY     = -100;                
SHARDTRACKER_ORIG_SOULBUTTONX       = 42;              
SHARDTRACKER_ORIG_SOULBUTTONY       = -128;              
SHARDTRACKER_ORIG_SPELLBUTTONX      = 70;              
SHARDTRACKER_ORIG_SPELLBUTTONY      = -128;

--[[ Original values backup
SHARDTRACKER_ORIG_SHARDBUTTONX      = -77;
SHARDTRACKER_ORIG_SHARDBUTTONY      = 25;
SHARDTRACKER_ORIG_HEALTHBUTTONX     = -81;              
SHARDTRACKER_ORIG_HEALTHBUTTONY     = 5;                
SHARDTRACKER_ORIG_SOULBUTTONX       = -80;              
SHARDTRACKER_ORIG_SOULBUTTONY       = -17;              
SHARDTRACKER_ORIG_SPELLBUTTONX      = -74;              
SHARDTRACKER_ORIG_SPELLBUTTONY      = -37;   
]]--



SHARDTRACKER_CNTR_SHARDBUTTONX      = -495;
SHARDTRACKER_CNTR_SHARDBUTTONY      = -250;
SHARDTRACKER_CNTR_HEALTHBUTTONX     = -438;
SHARDTRACKER_CNTR_HEALTHBUTTONY     = -251;
SHARDTRACKER_CNTR_SOULBUTTONX       = -495;
SHARDTRACKER_CNTR_SOULBUTTONY       = -300;
SHARDTRACKER_CNTR_SPELLBUTTONX      = -438;
SHARDTRACKER_CNTR_SPELLBUTTONY      = -300;

-- slash commands
SHARDTRACKER_DEBUG_CMD      = "debug";
SHARDTRACKER_TOGGLE_CMD     = "toggle";
SHARDTRACKER_ON_CMD         = "on";
SHARDTRACKER_OFF_CMD        = "off";
SHARDTRACKER_BAG_CMD        = "bag";
SHARDTRACKER_SORT_CMD       = "sort";
SHARDTRACKER_LIMIT_CMD      = "limit";
SHARDTRACKER_SOUND_CMD      = "sound";
SHARDTRACKER_MONITOR_CMD    = "monitor";
SHARDTRACKER_HELP_CMD       = "help";
SHARDTRACKER_FLASH_CMD      = "flash";
SHARDTRACKER_LOCK_CMD       = "lock";
SHARDTRACKER_UNLOCK_CMD     = "unlock";
SHARDTRACKER_INFO_CMD       = "info";
SHARDTRACKER_RESET_CMD      = "reset";
SHARDTRACKER_CENTER_CMD     = "center";
SHARDTRACKER_TOGGLE_CMD     = "toggle";
SHARDTRACKER_SCALE_CMD      = "scale";
SHARDTRACKER_SCALE_1_CMD    = "regular";
SHARDTRACKER_SCALE_2_CMD    = "large";
SHARDTRACKER_SCALE_3_CMD    = "biggie";
SHARDTRACKER_SCALE_4_CMD    = "supersizeme";
SHARDTRACKER_SOUL_POPUP_CMD = "soulpopup";
SHARDTRACKER_SHARDBG_CMD    = "shardbg";
SHARDTRACKER_RESTRICT_CMD   = "restrict";
SHARDTRACKER_ADD_CMD        = "add";
SHARDTRACKER_REMOVE_CMD     = "remove";
SHARDTRACKER_LIST_CMD       = "list";

SHARDTRACKER_CONFIG = { };
SHARDTRACKER_CONFIG.ENABLED                     = 1;      -- whether or not the plugin should be active
SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED         = 1;      -- whether or not the plugin should be active
SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED        = 1;      -- whether or not the plugin should be active
SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED          = 1;      -- whether or not the plugin should be active
SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED         = 1;      -- whether or not the plugin should be active
SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE           = 0;      -- whether or not to flash the healthstone button when we don't have a healthstone
SHARDTRACKER_CONFIG.USE_SOUND                   = 1;      -- whether or not to play sounds when soulstone cooldowns expire / players need new healthstones
SHARDTRACKER_CONFIG.THE_SORT_BAG                = 4;      -- the default bag to sort shards into
SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT    = 1;      -- when your total number of shards drops below this number, your shard button will flash
SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES  = 1;      -- whether or not to monitor the status of your party member's healthstones
SHARDTRACKER_CONFIG.BUTTONSCALE                 = SHARDTRACKER_SCALE_1_CMD;
SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP             = 1;
SHARDTRACKER_CONFIG.SHOW_SHARD_BG               = 1;
SHARDTRACKER_CONFIG.LIST_RESTRICT               = 0;
SHARDTRACKER_CONFIG.COMM_LIST                   = { };

-- locations of the various buttons on the interface
SHARDTRACKER_CONFIG.SHARDBUTTONX                = SHARDTRACKER_ORIG_SHARDBUTTONX;    
SHARDTRACKER_CONFIG.SHARDBUTTONY                = SHARDTRACKER_ORIG_SHARDBUTTONY;
SHARDTRACKER_CONFIG.HEALTHBUTTONX               = SHARDTRACKER_ORIG_HEALTHBUTTONX;
SHARDTRACKER_CONFIG.HEALTHBUTTONY               = SHARDTRACKER_ORIG_HEALTHBUTTONY;
SHARDTRACKER_CONFIG.SOULBUTTONX                 = SHARDTRACKER_ORIG_SOULBUTTONX;
SHARDTRACKER_CONFIG.SOULBUTTONY                 = SHARDTRACKER_ORIG_SOULBUTTONY;
SHARDTRACKER_CONFIG.SPELLBUTTONX                = SHARDTRACKER_ORIG_SPELLBUTTONX;
SHARDTRACKER_CONFIG.SPELLBUTTONY                = SHARDTRACKER_ORIG_SPELLBUTTONY;

SHARDTRACKER_COLOR_RED       = "|cffff0000";
SHARDTRACKER_COLOR_YELLOW    = "|cffffff00";
SHARDTRACKER_COLOR_VIOLET    = "|cffbb88dd";
SHARDTRACKER_COLOR_GREY      = "|caaaaaaaa";
SHARDTRACKER_COLOR_WHITE     = "|cffffffff";
SHARDTRACKER_COLOR_BLUE      = "|cff3366ff";
SHARDTRACKER_COLOR_GREEN     = "|cff64c528";
SHARDTRACKER_COLOR_LTGREEN   = "|cffE4FB75";
SHARDTRACKER_COLOR_CLOSE     = "|r";


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              INITIALIZATION FUNCTIONS                                      --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Handle the loading stuff when the plugin first loads
--
-----------------------------------------------------------------------------------
function ShardTracker_OnLoad()  

    ShardTracker_ShardButton       = getglobal("ShardTrackerMinimapButton");
    ShardTracker_HealthStoneButton = getglobal("ShardTrackerMinimapButtonHealth");
    ShardTracker_SoulStoneButton   = getglobal("ShardTrackerMinimapButtonSoul");
    ShardTracker_SpellStoneButton  = getglobal("ShardTrackerMinimapButtonSpell");
    ShardTracker_ShardCountLabel   = getglobal("ShardTrackerText");
    ShardTracker_SoulStoneText     = getglobal("ShardTrackerSoulText");
    ShardTracker_SpellStoneText    = getglobal("ShardTrackerSpellText");
    
    ShardTracker_SpellStoneText:SetText("");
    ShardTracker_ShardButton:Hide();
    ShardTracker_HealthStoneButton:Hide();
    ShardTracker_SoulStoneButton:Hide();
    
    getglobal("ShardTrackerSortFrame"):Hide();
    
    ShardTracker_FlashTime = 0;                         -- timer for flashing buttons
    ShardTracker_FlashState = 1;                        -- whether we're flashing on or off
    ShardTracker_SoulStoneExpired = 0;                  -- whether the cooldown on using a soulstone has expired
    ShardTracker_TotalShards = 0;                       -- number of shards in our inventory
    ShardTracker_HaveHealthStone = 0;                   -- if we have a healthstone in our inventory
    ShardTracker_HaveSoulStone = 0;                     -- if we have a soulstone in our inventory
    ShardTracker_SpellStoneInBag = 0;                   -- if we've created a spellstone, but haven't yet equipped it
    ShardTracker_HealthStoneLoc = {-1, -1};             -- the bag / slot where our healthstone is
    ShardTracker_SoulstoneLoc = {-1, -1};               -- the bag / slot where our soulstone is
    ShardTracker_SpellStoneLoc = {-1, -1};              -- the bag / slot where our spellstone is
    ShardTracker_SpellStoneReady = 0;                   -- whether a spellstone is ready to be used
    ShardTracker_SpellStoneEquipped = 0;                -- whether a spellstone is equipped
    ShardTracker_SpellStoneIsActive = 0;                -- whether the spellstone was just used to cast its effect on us
    ShardTracker_Timer = { };                           -- structure to time cooldowns
    ShardTracker_Timer.SoulstoneExpireTimeSeconds = 0;
    ShardTracker_Timer.SpellstoneTimeSeconds = 0
    ShardTracker_intoSeconds = nil;                     -- whether our soulstone cooldown timer is counting down minutes or seconds
    ShardTracker_UnderTwoMinutes = nil;                 -- whether our soulstone cooldown timer is under two minutes
    ShardTracker_LastMinute = 0;                        -- track the last minute that we checked the soulstone's cooldown
    ShardTracker_SpellTableBuilt = nil;                 -- whether we've built the list of create health/soulstone spells available to us
    ShardTracker_OffHandItem = nil;                     -- the item in our offhand, so we can reequip it once we use up a spellstone
    ShardTracker_HealthstoneFlashing = {0, 0, 0, 0};    -- track the flashing state of party member's healthstone buttons
    ShardTracker_SentHealthstoneNotification = 0;       -- whether we've already notified warlocks in the party that we need a new healthstone
    ShardTracker_CosmosEnabled = false;                 -- whether cosmos is enabled
    ShardTracker_MessageLanguage = "";                  -- the language to send our messages in
    ShardTracker_StopPinging = true;                    -- set when it's safe to stop pinging a party member about his/her healthstone
    ShardTracker_PingCounter = 0;                       -- remember how often we've pinged someone
    ShardTracker_ForceNonCosmosChat = false;            -- used to force chat into noncosmos mode for testing
    ShardTracker_DragLock = true;                       -- when true, we can't drag buttons around the interface
    ShardTracker_BeingDragged = false;                  -- true when we're in the midst of dragging a button
    ShardTracker_DisplayedLoginMessage = false;         -- track whether we've displayed the initial "shard tracker is running" message
    
    ShardTracker_PartyMemberList = { };                 -- this list tracks our party members and various info about them
                                                        -- needed because we get notified of members joining/leaving before GetNumPartyMembers() is

    this:RegisterEvent("VARIABLES_LOADED");
    this:RegisterEvent("PLAYER_ENTERING_WORLD");
    this:RegisterEvent("BAG_UPDATE");
    this:RegisterEvent("UNIT_NAME_UPDATE");
    this:RegisterEvent("SPELLCAST_START");
    this:RegisterEvent("SPELLS_CHANGED");
    this:RegisterEvent("LEARNED_SPELL_IN_TAB");
    this:RegisterEvent("UNIT_INVENTORY_CHANGED");
    this:RegisterEvent("PARTY_MEMBERS_CHANGED");
    
    -- shard sorting variables, some from ryu's ShardSort code
    ShardTracker_ShardSortFrameCounter = 0;
    ShardTracker_ShardMoverIndex = 0;
    ShardTracker_TotalShardsToMove = 5;
    ShardTracker_ShardMoverRunning = nil;
    ShardTracker_ShardMoverDone = false;
    ShardTracker_ShardMoverArray = { };
    ShardTracker_ShardMoverFilled = {nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil,nil};

    -- register with Cosmos
    ShardTracker_RegisterCosmos();
    
    -- setup chat watcher
    ShardTracker_SetupChatWatcher();
    
end

---------------------------------------------------------------------------------
--
-- Hook the chat onEvent handler to intercept incoming chat messages
-- 
---------------------------------------------------------------------------------
function ShardTracker_SetupChatWatcher()

    -- only do this if not running cosmos.  cosmos' handling is much cleaner
    if (not ShardTracker_CosmosEnabled) then  
       ShardTracker_Orig_ChatFrame_OnEvent = ChatFrame_OnEvent;
       ChatFrame_OnEvent = ShardTracker_ChatFrame_OnEvent;
    end
end



-----------------------------------------------------------------------------------
--
-- Workaround functions to handle a problem where the money field of a tooltip
-- will get hidden whenever tooltips are invoked.
-- 
-- Code by Telo.  From his post:
--   "Whenever a tooltip is set, the game sends a CLEAR_TOOLTIP event, which causes GameTooltip_OnEvent to call 
--   GameTooltip_ClearMoney, hiding the money frame for GameTooltip. There doesn't appear to be any identifying 
--   information sent with the CLEAR_TOOLTIP event that would allow GameTooltip_OnEvent to discriminate between 
--   its tooltip and others, so simply hooking this function doesn't appear to help. I've worked around the 
--   issue for my code by using something similar to the following code wherever I use a hidden tooltip."
--   (http://forums.worldofwarcraft.com/thread.aspx?fn=wow-interface-customization&t=16768&tmp=1#post16768)
--
-----------------------------------------------------------------------------------
local lOriginal_GameTooltip_ClearMoney;
local function ShardTracker_MoneyToggle()
    if( lOriginal_GameTooltip_ClearMoney ) then
        GameTooltip_ClearMoney = lOriginal_GameTooltip_ClearMoney;
        lOriginal_GameTooltip_ClearMoney = nil;
    else
        lOriginal_GameTooltip_ClearMoney = GameTooltip_ClearMoney;
        GameTooltip_ClearMoney = ShardTracker_GameTooltip_ClearMoney;
    end
    
    --Sea.wow.tooltip.protectTooltipMoney();
    --Sea.wow.tooltip.unprotectTooltipMoney();
end
function ShardTracker_GameTooltip_ClearMoney()
    -- Intentionally empty; don't clear money while we use hidden tooltips
end


---------------------------------------------------------------------------------
--
-- Register with Cosmos UI
-- 
---------------------------------------------------------------------------------
function ShardTracker_RegisterCosmos()

    --
    -- Add an entry to the Cosmos configuration menu
    --
    if (Cosmos_RegisterConfiguration) then
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER",
            "SECTION",
            SHARDTRACKER_CONFIG_HEADER,
            SHARDTRACKER_CONFIG_HEADER_INFO
        );
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_HEADER",
            "SEPARATOR",
            SHARDTRACKER_CONFIG_HEADER,
            SHARDTRACKER_CONFIG_HEADER_INFO
        );
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_ENABLED",
            "CHECKBOX",
            SHARDTRACKER_CONFIG_ENABLED,
            SHARDTRACKER_CONFIG_ENABLED_INFO,
            ShardTracker_Toggle,
            SHARDTRACKER_CONFIG.ENABLED
        );
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_FLASH_HEALTH",
            "CHECKBOX",
            SHARDTRACKER_CONFIG_FLASH_HEALTH,
            SHARDTRACKER_CONFIG_FLASH_HEALTH_INFO,
            ShardTracker_Toggle_Flash_Health,
            SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE
        );
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_SOUND",
            "CHECKBOX",
            SHARDTRACKER_CONFIG_SOUND,
            SHARDTRACKER_CONFIG_SOUND_INFO,
            ShardTracker_Toggle_Sound,
            SHARDTRACKER_CONFIG.USE_SOUND
        );
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_SOULPOP",
            "CHECKBOX",
            SHARDTRACKER_CONFIG_SOULPOP,
            SHARDTRACKER_CONFIG_SOULPOP_INFO,
            ShardTracker_Toggle_SoulPop,
            SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP
        );        
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_RESTRICT",
            "CHECKBOX",
            SHARDTRACKER_CONFIG_RESTRICT,
            SHARDTRACKER_CONFIG_RESTRICT_INFO,
            ShardTracker_Toggle_RestrictedChat,
            SHARDTRACKER_CONFIG.LIST_RESTRICT
        );        
                
        Cosmos_RegisterConfiguration(
            "COS_SHARDTRACKER_RESET",
            "BUTTON",
            SHARDTRACKER_RESET,
            SHARDTRACKER_RESET_INFO,
            ShardTracker_ResetButtonLocations,
            0,
            0,
            0,
            0,
            SHARDTRACKER_RESET_NAME
        );
       
        -- register our chat commands
        local ShardtrackerCommands = {"/shardtracker","/st"};
        Cosmos_RegisterChatCommand (
            "SHARDTRACKER_COMMANDS", -- Some Unique Group ID
            ShardtrackerCommands, -- The Commands
            Shardtracker_ChatCommandHandler,
            "" -- Description String
        );
        
        ShardTracker_CosmosEnabled = true;
        
    -- else enable the slash commands for non-cosmos users
    else
        SlashCmdList["SHARDTRACKER_SLASHENABLE"] = Shardtracker_ChatCommandHandler;
        SLASH_SHARDTRACKER_SLASHENABLE1 = "/shardtracker";
        SLASH_SHARDTRACKER_SLASHENABLE2 = "/st";
    end
    
    -- tell cosmos to watch for certain incoming chat messages
    if ( Cosmos_RegisterChatWatch ) then    
        Cosmos_RegisterChatWatch (
            "SHARDTRACKER", 
            {"WHISPER", "PARTY", "SYSTEM"}, 
            function (type, info, arg1, arg2, arg3, arg4) 
                if (type == "WHISPER" or type == "PARTY" or (type == "SYSTEM" and (string.find(arg1, "joins the party",1,true) or string.find(arg1, "leaves the party",1,true) or string.find(arg1, "Your group has been disbanded",1,true) or string.find(arg1, "You leave the group",1,true)))) then
                    return ShardTracker_ProcessMessage(arg1,arg2,type); 
                end
            end
            );
        Cosmos_RegisterChatWatch (
            "SHARDTRACKERCOSMOS", 
            {CHANNEL_PARTY}, 
            function (type, info, arg1, arg2, arg3, arg4) if (string.sub(arg1, 1, 4) == "<ST>") then ShardTracker_ProcessMessage(arg1,arg2,type); return 0; end end
            );
    end

    -- make sure we have the Print() function in case cosmos isn't installed
    if (not Print) then
        function Print(msg, r, g, b, frame) 
            if (not r) then r = 1.0; end
            if (not g) then g = 1.0; end
            if (not b) then b = 0.5; end
            if ( frame ) then 
                frame:AddMessage(msg,r,g,b);
            else
                if (DEFAULT_CHAT_FRAME) then 
                    DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
                end
            end
        end
    end

end


-----------------------------------------------------------------------------------
--
-- Run initialization stuff that needs to be done after we've loaded all the game variables
--
-----------------------------------------------------------------------------------
function ShardTracker_InitAfterVariablesLoaded()

    -- make sure older versions have their SavedVariables.lua updated to reflect any new data structures
    ShardTracker_UpdateOldConfigVersions();
    
    ShardTrackerDebug("Init after var loaded setting monitor healthstones");
    ShardTrackerDebug("Config info loaded: "..asText(SHARDTRACKER_CONFIG));
        
    -- position the buttons in the correct locations
    --ShardTracker_ButtonSetLocation();

    -- query the healthstone status of any party members
    ShardTracker_InitializePartyHealthstoneIcons();
    
    -- if using cosmos, schedule sending out a message to the party asking about their healthstones
    if (ShardTracker_CosmosEnabled) then
        Cosmos_Schedule(5,ShardTracker_SendHealthStoneQuery,"PARTY");
        for i = 1, GetNumPartyMembers() do

            -- if we both have cosmos, schedule the query
            if (Cosmos_IsCosmosUser(memberName)) then
                ShardTrackerDebug("Both "..tostring(UnitName("party"..i)).." and we have Cosmos, so start pinging membername = "..tostring(UnitName("party"..i)));
                ShardTracker_StartPingQuery(5, UnitName("party"..i), "WHISPER");

            -- if the party member doesn't have cosmos, send as normal msg
            else
                ShardTracker_SendHealthStoneQuery("WHISPER", UnitName("party"..i));
            end                  
        end

    -- otherwise we just send it right out
    else
        ShardTracker_SendHealthStoneQuery("PARTY");
    end
            
end


-----------------------------------------------------------------------------------
--
-- Update any outdated config files
--
-----------------------------------------------------------------------------------
function ShardTracker_UpdateOldConfigVersions()
    
    if (not SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES) then
        ShardTrackerDebug("Didn't find MONITOR_PARTY_HEALTHSTONES in config - added.");
        SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES = 1;
    end
    if (not SHARDTRACKER_CONFIG.SHARDBUTTONX) then
        ShardTrackerDebug("Didn't find button loc info in config - added.");
        SHARDTRACKER_CONFIG.SHARDBUTTONX                = -77;
        SHARDTRACKER_CONFIG.SHARDBUTTONY                = 25;
        SHARDTRACKER_CONFIG.HEALTHBUTTONX               = -81;
        SHARDTRACKER_CONFIG.HEALTHBUTTONY               = 5;
        SHARDTRACKER_CONFIG.SOULBUTTONX                 = -80;
        SHARDTRACKER_CONFIG.SOULBUTTONY                 = -17;
        SHARDTRACKER_CONFIG.SPELLBUTTONX                = -74;
        SHARDTRACKER_CONFIG.SPELLBUTTONY                = -37;
    end
    if (not SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED) then
        SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED         = 1;
        SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED        = 1;
        SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED          = 1;
        SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED         = 1;
    end
    if (not SHARDTRACKER_CONFIG.BUTTONSCALE) then
        SHARDTRACKER_CONFIG.BUTTONSCALE                 = SHARDTRACKER_SCALE_1_CMD;
    end
    if (not SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP) then
        SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP             = 1;
    end
    if (not SHARDTRACKER_CONFIG.SHOW_SHARD_BG) then
        SHARDTRACKER_CONFIG.SHOW_SHARD_BG               = 1;
    end
    if (not SHARDTRACKER_CONFIG.COMM_LIST) then
        SHARDTRACKER_CONFIG.COMM_LIST                   = { };
    end    
    if (not SHARDTRACKER_CONFIG.LIST_RESTRICT) then
        SHARDTRACKER_CONFIG.LIST_RESTRICT               = 0;
    end
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              EVENT HANDLING FUNCTIONS                                      --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Checks for events that might alter the shard count
--
-----------------------------------------------------------------------------------
function ShardTracker_OnEvent(event)

    -- once we've loaded into the game
    if (event == "VARIABLES_LOADED") then
        ShardTracker_InitAfterVariablesLoaded();
    
    -- if an event occurs that would affect the number of shards, soulstone, healthstone, or spellstone, update us
    elseif (UnitName("player") ~= "Unknown Being" and UnitName("player") ~= "Unknown Entity" and (event == "BAG_UPDATE" or event == "UNIT_NAME_UPDATE" or event == "PLAYER_ENTERING_WORLD" or event == "UNIT_INVENTORY_CHANGED") ) then
        ShardTracker_UpdateShardTracker();
        
    -- if we're using a soulstone on someone (casting Soulstone Resurrection), reset the soulstone timer
    elseif (event == "SPELLCAST_START") then
        if (arg1 == "Soulstone Resurrection") then
            ShardTracker_StartSoulstoneTimer();
        end
        
    -- if we learned new spells, update the list of spells we can cast
    -- since now we might be able to create a better healthstone or soulstone
    elseif (event == "LEARNED_SPELL_IN_TAB" or event == "SPELLS_CHANGED") then
        ShardTracker_BuildSpellTable();       
    end
end


---------------------------------------------------------------------------------
--
-- Break a slash command into its command and variable parts (i.e. "debug on"
-- breaks into command = "debug" and variable = "on"
-- 
---------------------------------------------------------------------------------
function ShardTracker_ParseCommand(msg)
    firstSpace = string.find(msg, " ", 1, true);
    if (firstSpace) then
        local command = string.sub(msg, 1, firstSpace - 1);
        local var  = string.sub(msg, firstSpace + 1);
        return command, var
    else
        return msg, "";
    end
end


---------------------------------------------------------------------------------
--
-- A simple slash command handler that can take commands in the form: "/slashCommand command var"
-- 
---------------------------------------------------------------------------------
function Shardtracker_ChatCommandHandler(msg)
    local command, var = ShardTracker_ParseCommand(msg);
    if ((not command) and msg) then
        command = msg;
    end
    if (command) then
        command = string.lower(command);
        if (command == SHARDTRACKER_DEBUG_CMD) then
            ShardTracker_ToggleDebug();
        elseif (command == SHARDTRACKER_ON_CMD) then
            ShardTracker_Toggle(1);
        elseif (command == SHARDTRACKER_OFF_CMD) then
            ShardTracker_Toggle(0);
        elseif (command == SHARDTRACKER_BAG_CMD) then
            ShardTracker_SetShardBag(var);
        elseif (command == SHARDTRACKER_SORT_CMD) then
            ShardTracker_SortShards();
        elseif (command == SHARDTRACKER_LIMIT_CMD) then
            ShardTracker_SetShardFlashLimit(var);
        elseif (command == SHARDTRACKER_MONITOR_CMD) then
            if (var) then
                if (var == "on") then
                    ShardTracker_Toggle_Monitor_Party_Healthstones(1, true);
                else
                    ShardTracker_Toggle_Monitor_Party_Healthstones(0, true);
                end
            else
                ShardTracker_Toggle_Monitor_Party_Healthstones(nil, true);
            end
        elseif (command == SHARDTRACKER_SOUND_CMD) then
            if (var == "on") then
                ShardTracker_Toggle_Sound(1);
            elseif (var == "off") then
                ShardTracker_Toggle_Sound(0);
            else
                ShardTracker_Toggle_Sound();
            end
        elseif (command == SHARDTRACKER_SHARDBG_CMD) then
            if (var == "on") then
                ShardTracker_Toggle_ShardBG(1);
            elseif (var == "off") then
                ShardTracker_Toggle_ShardBG(0);
            else
                ShardTracker_Toggle_ShardBG();
            end
        elseif (command == SHARDTRACKER_SOUL_POPUP_CMD) then
            if (var == "on") then
                ShardTracker_Toggle_SoulPop(1);
            elseif (var == "off") then
                ShardTracker_Toggle_SoulPop(0);
            else
                ShardTracker_Toggle_SoulPop();
            end
        elseif (command == SHARDTRACKER_RESTRICT_CMD) then
            if (var == "on") then
                ShardTracker_Toggle_RestrictedChat(1, true);
            elseif (var == "off") then
                ShardTracker_Toggle_RestrictedChat(0, true);
            else
                ShardTracker_Toggle_RestrictedChat(nil, true);
            end
        elseif (command == SHARDTRACKER_ADD_CMD) then
            ShardTracker_AddToCommList(var);
        elseif (command == SHARDTRACKER_REMOVE_CMD) then
            ShardTracker_RemoveFromCommList(var);
        elseif (command == SHARDTRACKER_LIST_CMD) then
            ShardTracker_ListCommList(var);
        elseif (command == SHARDTRACKER_FLASH_CMD) then
            if (var) then
                if (var == "on") then
                    ShardTracker_Toggle_Flash_Health(1, true);
                else
                    ShardTracker_Toggle_Flash_Health(0, true);
                end
            else
                ShardTracker_Toggle_Flash_Health(nil, true);
            end    
        elseif (command == SHARDTRACKER_LOCK_CMD or command == SHARDTRACKER_UNLOCK_CMD) then
            ShardTracker_ToggleLockedForDragging(command);
        elseif (command == "" or command == SHARDTRACKER_HELP_CMD or command == "?") then
            if (ChatFrame1) then ShardTracker_ShowHelp(); end
        elseif (command == SHARDTRACKER_INFO_CMD) then
            if (ChatFrame1) then ShardTracker_ShowInfoMessage(); end
        elseif (command == SHARDTRACKER_RESET_CMD) then
            ShardTracker_ResetButtonLocations();    
        elseif (command == SHARDTRACKER_CENTER_CMD) then
            ShardTracker_CenterButtonLocations();    
        elseif (command == SHARDTRACKER_TOGGLE_CMD) then        
            ShardTracker_ToggleSlashCommand(var);
        elseif (command == "buttons") then
            ShardTracker_PrintButtonLocs();
        elseif (command == "version") then
            ShardTrackerPrint("ShardTracker Version "..SHARDTRACKER_VERSION);
        elseif (command == "report") then
            ShardTracker_DebugPrintStatus();
        elseif (command == "dump") then
            ShardTracker_DebugDumpPartyList();
        elseif (command == "testyes" or command == "testno") then
            ShardTracker_DebugHealthstonePartyCode(command, var);
        elseif (command == "tron") then
            SHARDTRACKER_TRON = true;
            ShardTrackerPrint("ShardTracker TRON ON");
        elseif (command == "troff") then
            SHARDTRACKER_TRON = false;
            ShardTrackerPrint("ShardTracker TRON OFF");
        elseif (command == "notify" and var == "yes") then
            ShardTracker_SentHealthstoneNotification = 0;
            ShardTracker_NotifyHealthstoneStatus(SHARDTRACKER_GOT_HEALTHSTONE_MSG);
        elseif (command == "scale") then
            ShardTracker_SetButtonScale(var, true);
        elseif (command == "pop") then
            ShardTracker_NotifySoulstoneReady();
        elseif (command == "notify" and var == "no") then
            ShardTracker_SentHealthstoneNotification = 0;
            ShardTracker_NotifyHealthstoneStatus(SHARDTRACKER_NEED_HEALTHSTONE_MSG);
        elseif (command == "togglecosmos") then
            ShardTracker_ForceNonCosmosChat = not ShardTracker_ForceNonCosmosChat;
            ShardTrackerPrint("ShardTracker_ForceNonCosmosChat set to "..tostring(ShardTracker_ForceNonCosmosChat));
        elseif (command == "superhelp") then
            ShardTrackerPrint("report (print db report), tron/troff, testyes|testno <player> (set that player to having a HS), notify yes/no (notify warlocks that we need/have HS), togglecosmos (toggle cosmos msg sending)");
        elseif (command == "hello" and var == "sailor") then
            ShardTrackerPrint("Cretin!");
        elseif (command == "xyzzy" or command == "plugh") then
            ShardTrackerPrint("A hollow voice says 'Fool.'");
        elseif (command == "count" and var == "leaves") then
            ShardTrackerPrint("There are 69,105 leaves here.");
        end
    end
end

-----------------------------------------------------------------------------------
--
-- Set the number that, when your total shards drops below it, your shard button will flash
--
-----------------------------------------------------------------------------------
function ShardTracker_SetShardFlashLimit(var)

    if (UnitClass("player") == "Warlock") then    
        local badVar = false;
        if (not var) then
            badVar = true;
        else
            var = tonumber(var);
            if (not var) then
                badVar = true;
            elseif (var < 0 or var > 20) then
                badVar = true;
            end
        end

        if (not badVar) then
            SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT = var;
            ShardTrackerPrint("ShardTracker: Minimum number of shards before shard button flashes set to "..var..".");
            ShardTracker_UpdateShardButton(ShardTracker_TotalShards);
        else
            ShardTrackerPrint("Usage: /st limit [0-20]");
        end
    else
        ShardTrackerPrint("This function is only available to Warlocks.");
    end
end

--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              MAIN/UPDATE FUNCTIONS                                         --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Update the number of shards in the player's pack, as well as the
-- healthstone and soulstone status
--
-----------------------------------------------------------------------------------
function ShardTracker_UpdateShardTracker()
    local shardCount = 0;
    local foundSoulStone = 0;
    local foundHealthStone = 0;
    local foundSpellStone = 0;
    
    -- if this is our first update, build the table of known spells
    -- so we can cast create healthstone/soulstone spells
    if (not ShardTracker_SpellTableBuilt) then
        ShardTracker_BuildSpellTable();
        ShardTracker_SpellTableBuilt = true;
    end

    local theResult = { };

    -- now we're going to look through all our bags to see how many
    -- shards we have, whether we have a healthstone, and whether
    -- we have a soulstone
    
    -- for each bag
    for bag = 4, 0, -1 do
        local size = GetContainerNumSlots(bag);
        if (size > 0) then
        
            -- for each slot in the bag
            for slot=1, size, 1 do
            
                -- get info about the item in this slot
                local texture, itemCount = GetContainerItemInfo(bag, slot);
                if (itemCount) then
                    local itemName = ShardTracker_GetItemNameInBagSlot(bag, slot, false);
                                            
                    -- if the item has a name
                    if (itemName ~= "" and itemName ~= nil) then

                        -- if the item is a soul shard, increase the count
                        if (itemName == "Soul Shard") then
                            shardCount = shardCount+ 1;

                        -- if the item is a soulstone, remember which slot it's in so
                        -- that we can activate it later, if the player clicks on the healthstone button
                        elseif (string.find(itemName, "Soulstone", 1, TRUE) ~= nil) then
                            foundSoulStone = 1;
                            ShardTracker_SoulstoneLoc = {bag, slot};
                            ShardTracker_MoneyToggle();
                            theResult = Sea.wow.tooltip.scan("ShardTrackerTooltip");
                            ShardTracker_MoneyToggle();

                            -- look for the "Cooldown remaining" text in the stone's tooltip
                            timeRemainingText = "";
                            if (theResult) then
                                if (theResult[6]) then
                                    if (theResult[6].left) then              
                                        timeRemainingText = theResult[6].left;
                                    end
                                end
                            end
                            if (string.find(timeRemainingText, "Cooldown remaining", 1, TRUE) ~= nil) then
                                ShardTracker_SoulStoneExpired = 0;

                                -- grab the number of seconds or minutes remaining
                                for theTime in string.gfind(timeRemainingText, "%d+") do
                                    timeRemaining = theTime;
                                end

                                -- check if the tooltip has the word "sec" in it, we're counting down seconds
                                -- otherwise, we're counting down minutes
                                ShardTracker_intoSeconds = string.find(timeRemainingText, "sec", 1, TRUE);                                      
                                timeRemaining = tonumber(timeRemaining);

                                ShardTracker_ShowSoulstoneTime(timeRemaining, ShardTracker_intoSeconds);

                            -- else if there's no "cooldown remaining" then the cooldown has expired
                            else
                                if (ShardTracker_SoulStoneExpired ~= 1) then
                                    ShardTracker_NotifySoulstoneReady();
                                end
                                ShardTracker_SoulStoneExpired = 1;
                            end

                        -- if the item is a healthstone, remember which slot it's in so
                        -- that we can activate it later, if the player clicks on the healthstone button
                        elseif (string.find(itemName, "Healthstone", 1, TRUE) ~= nil) then
                            foundHealthStone = 1;
                            ShardTracker_HealthStoneLoc = {bag, slot};
                            
                        -- if the item is a spellstone
                        elseif (string.find(itemName, "Spellstone", 1, TRUE) ~= nil and (ShardTracker_SpellStoneEquipped ~= 1)) then
                            ShardTracker_SpellStoneInBag = 1;
                            ShardTracker_SpellStoneLoc = {bag, slot};
                        end
                    end
                end
            end
        end
    end
    
    -------------------------------------------------------------------------------------
    -- Now we're done looking through our pack, so we need to update our minimap buttons
    -------------------------------------------------------------------------------------
            
    -- if we're a warlock, we'll want to display the shard and soulstone buttons 
    if (UnitClass("player") == "Warlock") then
    
        --------------------------------
        -- Handle Shards
        --------------------------------
    
        ShardTracker_UpdateShardButton(shardCount);
        
        --------------------------------
        -- Handle Soulstone
        --------------------------------

        -- display the soulstone button, if we have one in inventory
        if (SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED == 1) then
            ShardTracker_SoulStoneButton:Show();
            ShardTracker_SoulStoneText:Show()
            
            -- if no soulstone in inventory, show the greyed out button
            if (foundSoulStone == 1) then
                ShardTrackerMinimapButtonSoul:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSoul");
            else
                ShardTracker_SoulstoneLoc = {-1, -1};
                ShardTrackerMinimapButtonSoul:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSoulGreyed");
                
                -- if we'd cast Soulstone Resurrection and haven't made a new Soulstone, track the time until we can cast it again
                if (ShardTracker_Timer.SoulstoneExpireTimeSeconds ~= 0) then
                    local timeRemaining = ShardTracker_Timer.SoulstoneExpireTimeSeconds - GetTime();
                    if (timeRemaining <= 0) then
                        ShardTracker_SoulStoneExpired = 1;
                        ShardTracker_Timer.SoulstoneExpireTimeSeconds = 0;
                        ShardTracker_NotifySoulstoneReady();
                    elseif (timeRemaining < 60) then
                        ShardTracker_intoSeconds = true;
                    elseif (timeRemaining < 120) then
                        ShardTracker_UnderTwoMinutes = true;
                    else
                        ShardTracker_SoulStoneExpired = 0;
                        ShardTracker_intoSeconds = nil;
                        ShardTracker_UnderTwoMinutes = nil;
                        timeRemaining = timeRemaining / 60;
                    end
                    ShardTracker_ShowSoulstoneTime(timeRemaining, ShardTracker_intoSeconds);
                end
            end
        else
            ShardTracker_SoulStoneButton:Hide();
            ShardTracker_SoulStoneText:Hide();
        end
        
        --------------------------------
        -- Handle Spellstone
        --------------------------------

        -- display the spellstone button, if we have one equipped
        if (SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED == 1) then
            local spellStoneInOffHand = 0;
            ShardTracker_SpellStoneButton:Show();
            ShardTracker_SpellStoneText:Show()

            -- see what we have in our secondary hand slot
            local id = GetInventorySlotInfo("SecondaryHandSlot");
            ShardTracker_MoneyToggle();
            local hasItem = getglobal("ShardTrackerTooltip"):SetInventoryItem("player", id);
            ShardTracker_MoneyToggle();

            -- if we're carrying an item in our secondary hand slot
            if (hasItem) then
                ShardTracker_MoneyToggle();
                local itemTip = Sea.wow.tooltip.scan("ShardTrackerTooltip");
                Sea.wow.tooltip.clear("ShardTrackerTooltip");
                ShardTracker_MoneyToggle();

                -- if that item has a tooltip description
                if (itemTip) then

                    -- if the item is a spellstone
                    local itemName = ShardTracker_GetItemName(itemTip);
                    if (string.find(itemName, "Spellstone", 1, true)) then

                        spellStoneInOffHand = 1;

                        -- look for the "Cooldown remaining" text in the stone's tooltip
                        timeRemainingText = itemTip[8].left;
                        if (string.find(timeRemainingText, "Cooldown remaining", 1, TRUE) ~= nil) then
                            ShardTracker_SpellStoneReady = 0;

                            -- grab the number of seconds remaining
                            for theTime in string.gfind(timeRemainingText, "%d+") do
                                timeRemaining = theTime;
                            end

                            -- check if the tooltip has the word "sec" in it, we're counting down seconds
                            -- otherwise, we're counting down minutes
                            timeRemaining = tonumber(timeRemaining);

                            ShardTracker_ShowSpellstoneTime(timeRemaining);

                        -- else if there's no "cooldown remaining" then the cooldown has expired and
                        -- the spellstone is ready for use
                        elseif (ShardTracker_SpellStoneIsActive == 0) then
                            ShardTracker_SpellStoneReady = 1;
                            ShardTracker_SpellStoneText:SetText("");
                            ShardTracker_SpellStoneText:Hide();
                        end
                    end
                end
            end
            
            -- if the spellstone is equipped, show the icon
            if (ShardTracker_SpellStoneReady == 1) then
                ShardTrackerMinimapButtonSpell:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSpellGreen");              
            elseif (ShardTracker_SpellStoneInBag == 1) then
                ShardTrackerMinimapButtonSpell:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSpell");              
            elseif (ShardTracker_SpellStoneText:GetText() ~= "" and ShardTracker_SpellStoneText:GetText() ~= nil) then
                ShardTrackerMinimapButtonSpell:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSpellGreyedHole");
            else
                ShardTrackerMinimapButtonSpell:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSpellGreyed");
            end
            
        -- else if spellstone is disabled, hide the button
        else
            ShardTracker_SpellStoneButton:Hide();
            ShardTracker_SpellStoneText:Hide();
        end

    -- else we're not a warlock, so hide the warlock-specific buttons (shard, soul, and spellstone)
    else
        ShardTracker_SoulStoneButton:Hide();
        ShardTracker_SoulStoneText:Hide();
        ShardTracker_SpellStoneButton:Hide();
        ShardTracker_SpellStoneText:Hide();
        ShardTracker_ShardCountLabel:Hide();
        ShardTracker_ShardButton:Hide();
    end 

    --------------------------------
    -- Handle Healthstone
    --------------------------------

    -- regardless of character class, display the healthstone button, if we have one in inventory
    if ((foundHealthStone == 1 and SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED == 1) or SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE == 1) then
        ShardTrackerMinimapButtonHealth:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardHealth");
        ShardTracker_HealthStoneButton:Show();
    else
        --ShardTracker_HealthStoneButton:Hide();
    end
        
    -- if we don't have a healthstone, reset the variable that tells us where it is in our pack
    if (foundHealthStone == 0) then
        ShardTracker_HealthStoneLoc = {-1, -1};
        ShardTrackerMinimapButtonHealth:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardHealthGreyed");
        
        -- if we previously had a healthstone, notify any warlocks that we now need a new one
        if (ShardTracker_HaveHealthStone == 1) then
            ShardTracker_SentHealthstoneNotification = 0;
            ShardTracker_NotifyHealthstoneStatus(SHARDTRACKER_NEED_HEALTHSTONE_MSG);
        end
        
    -- else if we just got a new healthstone, notify any Warlocks in our party that we now have a Healthstone
    elseif (foundHealthStone == 1 and ShardTracker_HaveHealthStone == 0) then
        ShardTracker_SentHealthstoneNotification = 0;
        ShardTracker_NotifyHealthstoneStatus(SHARDTRACKER_GOT_HEALTHSTONE_MSG);
    end
    
    -- store the total number of shards for later use
    ShardTracker_TotalShards = shardCount;
    ShardTracker_HaveHealthStone = foundHealthStone;
    ShardTracker_HaveSoulStone = foundSoulStone;
    
end


-----------------------------------------------------------------------------------
--
-- Update the flashing buttons and everything else that needs to update each frame
--
-----------------------------------------------------------------------------------
function ShardTracker_OnUpdate(elapsed)

    -- we have to do this here because we don't always have our class info at "VARIABLES_LOADED"
    if (UnitClass("player") ~= nil and not ShardTracker_DisplayedLoginMessage) then
        ShardTracker_DisplayedLoginMessage = true;

        -- setup the default for monitoring healthstones.  non-warlocks default to always 
        -- listening for healthstone msgs (dummy proof: we really don't want to have to tell every non-warlock
        -- to remember to turn this on)
        
        -- show or don't show the shardtracker interface
        if (not ShardTracker_CosmosEnabled) then ShardTracker_Toggle(SHARDTRACKER_CONFIG.ENABLED) end;          

        -- turn on or off monitoring of party healthstone status
        if (UnitClass("player") == "Warlock") then
            ShardTrackerDebug("MONITOR_PARTY_HEALTHSTONES = "..SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES);
            ShardTracker_Toggle_Monitor_Party_Healthstones(SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES, true);
        else
            ShardTracker_Toggle_Monitor_Party_Healthstones(1, nil);
        end    

        -- when we first login, if we're a warlock, print a message about our monitoring status
        if (UnitClass("player") == "Warlock") then
            if (SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES == 1 and SHARDTRACKER_CONFIG.ENABLED == 1) then
                ShardTrackerPrint("ShardTracker is monitoring your party's Healthstone status.  To disable monitoring, use the \"/st monior off\" command.");
            else
                ShardTrackerPrint("ShardTracker isn't monitoring your party's Healthstone status.  To enable monitoring, use the \"/st monior on\" command.");
            end
        elseif (SHARDTRACKER_CONFIG.ENABLED == 1) then
            ShardTrackerPrint("ShardTracker is active.");
        end
        
        -- initialize the scale of the buttons
        ShardTracker_InitButtonScales();
            
    end

    ShardTracker_FlashTime = ShardTracker_FlashTime - elapsed;
    if ( ShardTracker_FlashTime < 0 ) then
        local overtime = -ShardTracker_FlashTime;
        if ( ShardTracker_FlashState == 0 ) then
            ShardTracker_FlashState = 1;
            ShardTracker_FlashTime = SHARDTRACKER_FLASH_TIME_ON;
        else
            ShardTracker_FlashState = 0;
            ShardTracker_FlashTime = SHARDTRACKER_FLASH_TIME_OFF;
        end
        if ( overtime < ShardTracker_FlashTime ) then
            ShardTracker_FlashTime = ShardTracker_FlashTime - overtime;
        end
    end

    -- flash the healthstone button
    if (SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE == 1) then
        if (ShardTracker_HaveHealthStone == 0) then
            if ( ShardTracker_FlashState == 1 ) then
                shardAlphaValue = (SHARDTRACKER_FLASH_TIME_ON - ShardTracker_FlashTime) / SHARDTRACKER_FLASH_TIME_ON;
                shardAlphaValue = shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA) + SHARDTRACKER_MIN_ALPHA;
            else
                shardAlphaValue = ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON;
                shardAlphaValue = (shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA)) + SHARDTRACKER_MIN_ALPHA;
                ShardTracker_HealthStoneButton:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
            end
            ShardTracker_HealthStoneButton:SetAlpha(shardAlphaValue);
        else
            ShardTracker_HealthStoneButton:SetAlpha(1.0);
        end
    end
        
    if (UnitClass("player") == "Warlock") then
    
        -- flash the soulstone button
        if (SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED == 1) then
            if (ShardTracker_SoulStoneExpired == 1) then
                if ( ShardTracker_FlashState == 1 ) then
                    shardAlphaValue = (SHARDTRACKER_FLASH_TIME_ON - ShardTracker_FlashTime) / SHARDTRACKER_FLASH_TIME_ON;
                    shardAlphaValue = shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA) + SHARDTRACKER_MIN_ALPHA;
                else
                    shardAlphaValue = ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON;
                    shardAlphaValue = (shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA)) + SHARDTRACKER_MIN_ALPHA;
                    ShardTracker_SoulStoneButton:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                end
                ShardTracker_SoulStoneButton:SetAlpha(shardAlphaValue);
            else
                ShardTracker_SoulStoneButton:SetAlpha(1.0);
            end
        end
        
        -- flash the total shards button
        if (SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED == 1) then
            if (SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT == nil) then SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT = 1; end
            if (ShardTracker_TotalShards < SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT) then
                if ( ShardTracker_FlashState == 1 ) then
                    shardAlphaValue = (SHARDTRACKER_FLASH_TIME_ON - ShardTracker_FlashTime) / SHARDTRACKER_FLASH_TIME_ON;
                    shardAlphaValue = shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA) + SHARDTRACKER_MIN_ALPHA;
                else
                    shardAlphaValue = ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON;
                    shardAlphaValue = (shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA)) + SHARDTRACKER_MIN_ALPHA;
                    ShardTracker_ShardButton:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                    ShardTracker_ShardCountLabel:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                end
                ShardTracker_ShardButton:SetAlpha(shardAlphaValue);
                ShardTracker_ShardButton:Show();
                ShardTracker_ShardCountLabel:SetAlpha(shardAlphaValue);
                ShardTracker_ShardCountLabel:Show();
            else
                ShardTracker_ShardButton:SetAlpha(1.0);
                ShardTracker_ShardCountLabel:SetAlpha(1.0);
            end
        end
        
        -- update the visibility of the party member healthstone icons
        --if (ShardTracker_CosmosEnabled) then
            for i = 1, table.getn(ShardTracker_PartyMemberList) do
                partyNumber = ShardTracker_GetPartyMemberNumber(ShardTracker_PartyMemberList[i].name);
                --ShardTrackerDebug("Updating partyNumber "..partyNumber.."s healthstone visibility.....");

                local healthStoneIcon = getglobal("PartyMemberFrame"..partyNumber.."ShardTrackerIcon");
                if (healthStoneIcon) then
                    --ShardTrackerDebug("   Found healthStoneIcon");
                    if (ShardTracker_PartyMemberList[i].hasHealthstone or ShardTracker_PartyMemberList[i].flashing) then
                        if (ShardTracker_PartyMemberList[i].hasHealthstone) then
                            --ShardTrackerDebug("   Has a healthStone");
                        else
                            --ShardTrackerDebug("   Doesn't have a healthStone.  Flashing is true.");
                        end
                        --ShardTrackerDebug("   Showing healthStoneIcon");
                        healthStoneIcon:Show();
                    else
                        --ShardTrackerDebug("   Hiding healthStoneIcon");
                        healthStoneIcon:Hide();
                    end
                else
                    --ShardTrackerDebug("   Didn't find healthStoneIcon!");
                end
            end

            -- flash the party members' healthstone buttons
            ShardTracker_Party1Healthstone = getglobal("PartyMemberFrame1ShardTrackerIcon");
            ShardTracker_Party2Healthstone = getglobal("PartyMemberFrame2ShardTrackerIcon");
            ShardTracker_Party3Healthstone = getglobal("PartyMemberFrame3ShardTrackerIcon");
            ShardTracker_Party4Healthstone = getglobal("PartyMemberFrame4ShardTrackerIcon");

            -- reset the flashing array
            for i = 1, 4 do
                ShardTracker_HealthstoneFlashing[i] = 0;
            end
            for i = 1, table.getn(ShardTracker_PartyMemberList) do
                partyNumber = ShardTracker_GetPartyMemberNumber(ShardTracker_PartyMemberList[i].name);
                if (ShardTracker_PartyMemberList[i].flashing) then
                    ShardTracker_HealthstoneFlashing[partyNumber] = 1;
                end
            end
            
            if (ShardTracker_Party1Healthstone) then
                if ( ShardTracker_FlashState == 1 ) then
                    shardAlphaValue = (SHARDTRACKER_FLASH_TIME_ON - ShardTracker_FlashTime) / SHARDTRACKER_FLASH_TIME_ON;
                    shardAlphaValue = shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA) + SHARDTRACKER_MIN_ALPHA;
                else
                    shardAlphaValue = ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON;
                    shardAlphaValue = (shardAlphaValue * (1 - SHARDTRACKER_MIN_ALPHA)) + SHARDTRACKER_MIN_ALPHA;
                    if (ShardTracker_Party1Healthstone and ShardTracker_HealthstoneFlashing[1] == 1) then
                        ShardTracker_Party1Healthstone:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                    end
                    if (ShardTracker_Party2Healthstone and ShardTracker_HealthstoneFlashing[2] == 1) then
                        ShardTracker_Party2Healthstone:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                    end
                    if (ShardTracker_Party3Healthstone and ShardTracker_HealthstoneFlashing[3] == 1) then
                        ShardTracker_Party3Healthstone:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                    end
                    if (ShardTracker_Party4Healthstone and ShardTracker_HealthstoneFlashing[4] == 1) then
                        ShardTracker_Party4Healthstone:SetAlpha(ShardTracker_FlashTime / SHARDTRACKER_FLASH_TIME_ON);
                    end
                end
            end
            if (ShardTracker_Party1Healthstone) then
                if (ShardTracker_HealthstoneFlashing[1] == 1) then
                    ShardTracker_Party1Healthstone:SetAlpha(shardAlphaValue);
                else
                    ShardTracker_Party1Healthstone:SetAlpha(1.0);
                end
            end
            if (ShardTracker_Party2Healthstone) then
                if (ShardTracker_HealthstoneFlashing[2] == 1) then
                    ShardTracker_Party2Healthstone:SetAlpha(shardAlphaValue);
                else
                    ShardTracker_Party2Healthstone:SetAlpha(1.0);
                end
            end
            if (ShardTracker_Party3Healthstone) then
                if (ShardTracker_HealthstoneFlashing[3] == 1) then
                    ShardTracker_Party3Healthstone:SetAlpha(shardAlphaValue);
                else
                    ShardTracker_Party3Healthstone:SetAlpha(1.0);
                end 
            end
            if (ShardTracker_Party4Healthstone) then
                if (ShardTracker_HealthstoneFlashing[4] == 1) then
                    ShardTracker_Party4Healthstone:SetAlpha(shardAlphaValue);
                else
                    ShardTracker_Party4Healthstone:SetAlpha(1.0);
                end
            end
        --end
        
        -- if a minute has passed, or we're counting down seconds on the soulstone, or counting down seconds on the spellstone, update the shard tracker
        local hour, minute = GetGameTime();
        if (minute ~= ShardTracker_LastMinute or ShardTracker_intoSeconds or ShardTracker_UnderTwoMinutes or (ShardTracker_SpellStoneEquipped == 1 and ShardTracker_SpellStoneReady == 0)) then
            ShardTracker_LastMinute = minute;
            ShardTracker_UpdateShardTracker();
        end

        -- see if we still need to ping non-responsive party members
        if (not ShardTracker_StopPinging) then
            ShardTracker_PingCounter = math.mod(ShardTracker_PingCounter + 1,60);
            if (ShardTracker_PingCounter == 0) then
                ShardTrackerDebug("Calling ShardTracker_UpdatePartyMemberPingStatus from OnUpdate",0,1,0);
                ShardTracker_UpdatePartyMemberPingStatus();
            end
        end

        -- update the shard sorting code
        ShardTracker_UpdateShardSorter();
        
    end
end


function ShardTracker_UpdateShardButton(shardCount)

    local limitNum;
    
    -- display the number of shards
    ShardTracker_ShardCountLabel:SetText(shardCount);

    -- if the number of shards is less than the limit, display the number in yellow    
    if (shardCount < SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT) then
        ShardTracker_ShardCountLabel:SetTextColor(0.89, 0.98, 0.46);
        if (SHARDTRACKER_CONFIG.SHOW_SHARD_BG) then
            ShardTrackerMinimapButton:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardRed");
        else
            ShardTrackerMinimapButton:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardBlankRed");
        end
    else
        ShardTracker_ShardCountLabel:SetTextColor(0.39, 1.0, 0.0);
        if (SHARDTRACKER_CONFIG.SHOW_SHARD_BG) then
            ShardTrackerMinimapButton:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\Shard");
        else
            ShardTrackerMinimapButton:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardBlank");        
        end
    end
    if (SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED == 1) then
        ShardTracker_ShardCountLabel:Show();
        ShardTracker_ShardButton:Show();
    else
        ShardTracker_ShardCountLabel:Hide();
        ShardTracker_ShardButton:Hide();
    end
end        


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              SOULSTONE FUNCTIONS                                   --
--                                                                                            --
--============================================================================================--
--============================================================================================--

-----------------------------------------------------------------------------------
--
-- Create a new soulstone if we click on our soulstone button
--
-----------------------------------------------------------------------------------
function ShardTracker_OnSoulButtonClick()
    local myClass = UnitClass("player");
    if (myClass == "Warlock") then
        if (ShardTracker_HaveSoulStone == 1) then
            if (ShardTracker_SoulstoneLoc[1] ~= -1 and ShardTracker_SoulstoneLoc[2] ~= -1) then
                UseContainerItem(ShardTracker_SoulstoneLoc[1], ShardTracker_SoulstoneLoc[2]);
            end
        elseif (ShardTracker_TotalShards > 0 and ShardTracker_CreateSoulStoneSpellID) then
            CastSpell(ShardTracker_CreateSoulStoneSpellID, "spell");
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Remember when the soulstone will expire so that we can countdown the minutes/seconds
-- until we need to cast another one.
--
-----------------------------------------------------------------------------------
function ShardTracker_StartSoulstoneTimer()
    ShardTracker_Timer.SoulstoneExpireTimeSeconds = GetTime() + (60 * 30);
    ShardTracker_intoSeconds = nil;
    ShardTracker_UnderTwoMinutes = nil;
end


-----------------------------------------------------------------------------------
--
-- Show the time remaining until we can cast another soulstone
--
-----------------------------------------------------------------------------------
function ShardTracker_ShowSoulstoneTime(timeRemaining, intoSeconds)
                                    
    -- display the number of minutes or seconds remaining until
    -- the player can use another soulstone
    timeRemaining = Sea.math.round(timeRemaining);
    if (timeRemaining > 0) then
        ShardTracker_SoulStoneText:SetText(timeRemaining);
        ShardTracker_SoulStoneText:Show();
    else
        ShardTracker_SoulStoneText:SetText("");
        ShardTracker_SoulStoneText:Hide();
    end
    
    -- if we're displaying seconds (as opposed to minutes), show
    -- the seconds in yellow text
    if (intoSeconds ~= nil) then
        ShardTracker_SoulStoneText:SetTextColor(1.0,1.0,1.0);
    else
        ShardTracker_SoulStoneText:SetTextColor(0.89, 0.98, 0.46);
    end
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              HEALTHSTONE FUNCTIONS                                     --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- If the user clicks on the healthstone button, activate the healthstone.
-- If there's no healthstone to use, try to create one.
--
-----------------------------------------------------------------------------------
function ShardTracker_OnHealthButtonClick()
    
    -- if we found a healthstone in a bag and slot
    if (ShardTracker_HealthStoneLoc[1] ~= -1 and ShardTracker_HealthStoneLoc[2] ~= -1) then
    
        -- if we're a warlock and have a friendly target selected, try to give the healthstone to that target
        if (UnitClass("player") == "Warlock" and UnitExists("target") and UnitIsPlayer("target") and UnitCanCooperate("player", "target")) then
        
            -- pickup the healthstone and drop it on the target
            PickupContainerItem(ShardTracker_HealthStoneLoc[1], ShardTracker_HealthStoneLoc[2]);
            if ( CursorHasItem() ) then
                DropItemOnUnit("target");
            end
            
        -- otherwise without a friendly target selected, use the healthstone
        else
            UseContainerItem(ShardTracker_HealthStoneLoc[1], ShardTracker_HealthStoneLoc[2]);
        
            -- notify any Warlocks in our party that we need another Healthstone
            ShardTracker_NotifyHealthstoneStatus(SHARDTRACKER_NEED_HEALTHSTONE_MSG);
        end
        
    -- else we don't have a healthstone, so if we're a warlock, try to create one
    else
        if (UnitClass("player") == "Warlock") then
            if (ShardTracker_TotalShards > 0 and ShardTracker_CreateHealthStoneSpellID) then
                CastSpell(ShardTracker_CreateHealthStoneSpellID, "spell");
            end
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Create a new healthstone if we click on a party member's healthstone button
--
-----------------------------------------------------------------------------------
function ShardTracker_PartyHealthstoneButtonClick()
    local myClass = UnitClass("player");
    if (myClass == "Warlock") then
        if (ShardTracker_TotalShards > 0 and ShardTracker_CreateHealthStoneSpellID) then
            CastSpell(ShardTracker_CreateHealthStoneSpellID, "spell");
        end
    end
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              SPELLSTONE FUNCTIONS                                      --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Activate a spellstone 
--
-----------------------------------------------------------------------------------
function ShardTracker_ActivateSpellStone()

    -- use the spellstone
    UseInventoryItem(GetInventorySlotInfo("SecondaryHandSlot"));    

    -- re-equip the old offhanditem, if any
    if (ShardTracker_OffHandItem) then
        ShardTrackerDebug("ShardTracker_ActivateSpellStone: looking for previous offhand item = "..ShardTracker_OffHandItem);
        theBag, theSlot = ShardTrackerFindItemBagSlotByName(ShardTracker_OffHandItem);
        if (theBag) then    
            ShardTrackerDebug("ShardTracker_ActivateSpellStone: found offhand item at bag =  "..theBag.." slot = "..theSlot);
            UseContainerItem(theBag, theSlot);
        else
            ShardTrackerDebug("ShardTracker_ActivateSpellStone: couldn't find offhand item!");
        end
    end
    
    -- reset the ShardTracker_SpellStoneEquipped and ShardTracker_SpellStoneReady and ShardTracker_SpellStoneInBag flags
    ShardTracker_SpellStoneEquipped = 0;
    ShardTracker_SpellStoneReady = 0;
    ShardTracker_SpellStoneIsActive = 1;
    ShardTracker_SpellStoneInBag = 0;
    ShardTrackerMinimapButtonSpell:SetNormalTexture("Interface\\AddOns\\ShardTracker\\Images\\ShardSpellGreyed");
end


-----------------------------------------------------------------------------------
--
-- Create a new spellstone if we click on our spellstone button, or use our spellstone
--
-----------------------------------------------------------------------------------
function ShardTracker_OnSpellButtonClick()
    local myClass = UnitClass("player");
    if (myClass == "Warlock") then
    
        -- if a spellstone is sitting in our bag ready to be equipped
        if (ShardTracker_SpellStoneInBag == 1 and ShardTracker_SpellStoneEquipped ~= 1) then
            ShardTracker_EquipSpellStone();
            
        -- if we have a spellstone equipped and it's ready for use
        elseif (ShardTracker_SpellStoneEquipped == 1 and ShardTracker_SpellStoneReady == 1) then
            ShardTrackerDebug("ShardTracker_OnSpellButtonClick: trying to use spellstone!");
            ShardTracker_ActivateSpellStone();

        -- else if we have shards and know how to create a spellstone, try to create a spellstone
        elseif (ShardTracker_TotalShards > 0 and ShardTracker_CreateSpellStoneSpellID and (ShardTracker_SpellStoneEquipped ~= 1)) then
            ShardTrackerDebug("Trying to cast create spellstone, spell #"..ShardTracker_CreateSpellStoneSpellID);
            CastSpell(ShardTracker_CreateSpellStoneSpellID, "spell");
            ShardTracker_SpellStoneIsActive = 0;
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Equip a spellstone
--
-----------------------------------------------------------------------------------
function ShardTracker_EquipSpellStone()

    local theResult = true;
    
    -- if we're wielding a 2H weapon, we need to unequip it first
    local twoHandedWeapon = ShardTracker_IsWeaponTwoHanded();
    if (twoHandedWeapon) then
        PickupInventoryItem(GetInventorySlotInfo("MainHandSlot"));
        theResult = ShardTracker_BagItem();
    end

    -- if we don't have 2H weapon issues
    if (theResult) then    
    
        -- remember what item (if any) we have equipped in our offhand
        -- so that we can re-equip when we use the spellstone
        ShardTracker_RememberOffHandItem(twoHandedWeapon);

        -- equip the spellStone
        PickupContainerItem(ShardTracker_SpellStoneLoc[1], ShardTracker_SpellStoneLoc[2]);
        ShardTrackerDebug("Picking up spellstone at bag = "..ShardTracker_SpellStoneLoc[1].." slot = "..ShardTracker_SpellStoneLoc[2]);
        if ( CursorHasItem() ) then
            ShardTrackerDebug("Cursor has item.  Trying to equip spellstone...");
            PickupInventoryItem(18);
        else
            ShardTrackerDebug("Cursor doesn't have item!");
        end

        -- we've equipped it, but it's not ready to be used yet (have to wait out the cooldown)
        ShardTracker_SpellStoneEquipped = 1;
        ShardTracker_SpellStoneReady = 0;
        ShardTracker_SpellStoneInBag = 0;
        ShardTracker_SpellStoneLoc = {-1, -1};
    else
        ShardTrackerPrint("ShardTracker: Unable to equip Spellstone!  Perhaps there's not enough pack space to unequip your mainhand weapon?");
    end
    
end


-----------------------------------------------------------------------------------
--
-- Store what we have equipped in our offhand so we can reequip it after
-- using a spellstone
--
-----------------------------------------------------------------------------------
function ShardTracker_RememberOffHandItem(isTwoHanded)
    ShardTracker_OffHandItem = nil;
    
    local id = 0;
    if (isTwoHanded) then
        id = GetInventorySlotInfo("MainHandSlot");
    else
        id = GetInventorySlotInfo("SecondaryHandSlot");
    end
    
    ShardTracker_MoneyToggle();
    local hasItem = getglobal("ShardTrackerTooltip"):SetInventoryItem("player", id);
    ShardTracker_MoneyToggle();
    if (hasItem) then
        ShardTracker_MoneyToggle();
        local itemTip = Sea.wow.tooltip.scan("ShardTrackerTooltip");
        Sea.wow.tooltip.clear("ShardTrackerTooltip");
        ShardTracker_MoneyToggle();
        if (itemTip) then
            ShardTracker_OffHandItem = ShardTracker_GetItemName(itemTip);
            ShardTrackerDebug("ShardTracker_RememberOffHandItem: ShardTracker_OffHandItem = "..ShardTracker_OffHandItem);
            if (ShardTracker_OffHandItem == "NIL") then ShardTracker_OffHandItem = nil end;
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Show the time remaining until we can use a spellstone
--
-----------------------------------------------------------------------------------
function ShardTracker_ShowSpellstoneTime(timeRemaining)
                                    
    -- display the number of seconds remaining until
    -- the player can use the spellstone
    timeRemaining = Sea.math.round(timeRemaining);
    if (timeRemaining > 0) then
        ShardTracker_SpellStoneText:SetText(timeRemaining);
        ShardTracker_SpellStoneText:Show();
    else
        ShardTracker_SpellStoneText:SetText("");
        ShardTracker_SpellStoneText:Hide();
    end
    
    ShardTracker_SpellStoneText:SetTextColor(0.7, 0.95, 0.98);
end



--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              INTERFACE FUNCTIONS                                           --
--                                                                                            --
--============================================================================================--
--============================================================================================--

---------------------------------------------------------------------------------
--
-- Start dragging the shard button
-- 
---------------------------------------------------------------------------------
function ShardTracker_OnDragStart()
    if (not ShardTracker_DragLock and not ShardTracker_BeingDragged) then
        this:StartMoving();
        ShardTracker_BeingDragged = true;
    else
        ShardTrackerPrint("To reposition the ShardTracker buttons, you must first unlock them with the \"/st unlock\" command.");
    end
end


---------------------------------------------------------------------------------
--
-- Stop dragging the shard button
-- 
---------------------------------------------------------------------------------
function ShardTracker_OnDragStop()
    if (ShardTracker_BeingDragged) then
        this:StopMovingOrSizing()
        ShardTracker_BeingDragged = false;
        local x,y = this:GetCenter();
        local px,py = this:GetParent():GetCenter();
        local ox = x-px;
        local oy = y-py;
        ShardTrackerDebug(" ox = "..ox.." oy = "..oy);
        ShardTrackerDebug(" Name = "..this:GetName());
        ShardTracker_StoreButtonLocation(this:GetName(), ox, oy);
    end
end


---------------------------------------------------------------------------------
--
-- Register the button for drag
-- 
---------------------------------------------------------------------------------
function ShardTracker_ButtonOnLoad()
    this:RegisterForDrag("LeftButton");
end


---------------------------------------------------------------------------------
--
-- Toggle the lock / unlock status of the interface
-- 
---------------------------------------------------------------------------------
function ShardTracker_ToggleLockedForDragging(theCommand)
    if (theCommand == SHARDTRACKER_LOCK_CMD) then
        ShardTracker_DragLock = true;
        ShardTrackerPrint("ShardTracker buttons are now locked in place and functional.  To reposition them, unlock them with the \"/st unlock\" command.");
    elseif (theCommand == SHARDTRACKER_UNLOCK_CMD) then
        ShardTracker_DragLock = false;
        ShardTrackerPrint("ShardTracker buttons are now unlocked.  You may position them anywhere on the screen.  While unlocked, they are NOT FUNCTIONAL.  To lock them in place, use the \"/st lock\" command.");
    end
end


---------------------------------------------------------------------------------
--
-- Store the location of the specified button, relative to its parent in the XML
-- 
---------------------------------------------------------------------------------
function ShardTracker_StoreButtonLocation(buttonName, ox, oy)

    if (buttonName == "ShardTrackerMinimapButton") then
        SHARDTRACKER_CONFIG.SHARDBUTTONX = ox;
        SHARDTRACKER_CONFIG.SHARDBUTTONY = oy;
    elseif (buttonName == "ShardTrackerMinimapButtonHealth") then
        SHARDTRACKER_CONFIG.HEALTHBUTTONX = ox;
        SHARDTRACKER_CONFIG.HEALTHBUTTONY = oy;
    elseif (buttonName == "ShardTrackerMinimapButtonSoul") then
        SHARDTRACKER_CONFIG.SOULBUTTONX = ox;
        SHARDTRACKER_CONFIG.SOULBUTTONY = oy;
    elseif (buttonName == "ShardTrackerMinimapButtonSpell") then
        SHARDTRACKER_CONFIG.SPELLBUTTONX = ox;
        SHARDTRACKER_CONFIG.SPELLBUTTONY = oy;
    end    
end


---------------------------------------------------------------------------------
--
-- Move the interface buttons to the stored locations
-- 
---------------------------------------------------------------------------------
function ShardTracker_ButtonSetLocation()

    ShardTrackerMinimapButton:ClearAllPoints();
    ShardTrackerMinimapButtonHealth:ClearAllPoints();
    ShardTrackerMinimapButtonSoul:ClearAllPoints();
    ShardTrackerMinimapButtonSpell:ClearAllPoints();

    ShardTrackerMinimapButton:SetPoint("CENTER", ShardTrackerMinimapButton:GetParent():GetName(), "CENTER", SHARDTRACKER_CONFIG.SHARDBUTTONX, SHARDTRACKER_CONFIG.SHARDBUTTONY);
    ShardTrackerMinimapButtonHealth:SetPoint("CENTER", ShardTrackerMinimapButtonHealth:GetParent():GetName(), "CENTER", SHARDTRACKER_CONFIG.HEALTHBUTTONX, SHARDTRACKER_CONFIG.HEALTHBUTTONY);
    ShardTrackerMinimapButtonSoul:SetPoint("CENTER", ShardTrackerMinimapButtonSoul:GetParent():GetName(), "CENTER", SHARDTRACKER_CONFIG.SOULBUTTONX, SHARDTRACKER_CONFIG.SOULBUTTONY);
    ShardTrackerMinimapButtonSpell:SetPoint("CENTER", ShardTrackerMinimapButtonSpell:GetParent():GetName(), "CENTER", SHARDTRACKER_CONFIG.SPELLBUTTONX, SHARDTRACKER_CONFIG.SPELLBUTTONY);
end


---------------------------------------------------------------------------------
--
-- Move the interface buttons back to their default locations
-- 
---------------------------------------------------------------------------------
function ShardTracker_ResetButtonLocations(centerButtons)

    ShardTrackerPrint("Button locations reset.");
    SHARDTRACKER_CONFIG.SHARDBUTTONX                = SHARDTRACKER_ORIG_SHARDBUTTONX;    
    SHARDTRACKER_CONFIG.SHARDBUTTONY                = SHARDTRACKER_ORIG_SHARDBUTTONY;
    SHARDTRACKER_CONFIG.HEALTHBUTTONX               = SHARDTRACKER_ORIG_HEALTHBUTTONX;
    SHARDTRACKER_CONFIG.HEALTHBUTTONY               = SHARDTRACKER_ORIG_HEALTHBUTTONY;
    SHARDTRACKER_CONFIG.SOULBUTTONX                 = SHARDTRACKER_ORIG_SOULBUTTONX;
    SHARDTRACKER_CONFIG.SOULBUTTONY                 = SHARDTRACKER_ORIG_SOULBUTTONY;
    SHARDTRACKER_CONFIG.SPELLBUTTONX                = SHARDTRACKER_ORIG_SPELLBUTTONX;
    SHARDTRACKER_CONFIG.SPELLBUTTONY                = SHARDTRACKER_ORIG_SPELLBUTTONY;
    ShardTracker_ButtonSetLocation();
end


---------------------------------------------------------------------------------
--
-- Move the interface buttons to the center of the screen
-- 
---------------------------------------------------------------------------------
function ShardTracker_CenterButtonLocations()

    ShardTrackerPrint("Button locations set to center of screen.");
    SHARDTRACKER_CONFIG.SHARDBUTTONX                = SHARDTRACKER_CNTR_SHARDBUTTONX;    
    SHARDTRACKER_CONFIG.SHARDBUTTONY                = SHARDTRACKER_CNTR_SHARDBUTTONY;
    SHARDTRACKER_CONFIG.HEALTHBUTTONX               = SHARDTRACKER_CNTR_HEALTHBUTTONX;
    SHARDTRACKER_CONFIG.HEALTHBUTTONY               = SHARDTRACKER_CNTR_HEALTHBUTTONY;
    SHARDTRACKER_CONFIG.SOULBUTTONX                 = SHARDTRACKER_CNTR_SOULBUTTONX;
    SHARDTRACKER_CONFIG.SOULBUTTONY                 = SHARDTRACKER_CNTR_SOULBUTTONY;
    SHARDTRACKER_CONFIG.SPELLBUTTONX                = SHARDTRACKER_CNTR_SPELLBUTTONX;
    SHARDTRACKER_CONFIG.SPELLBUTTONY                = SHARDTRACKER_CNTR_SPELLBUTTONY;
    ShardTracker_ButtonSetLocation();
end


function ShardTracker_PrintButtonLocs()
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SHARDBUTTONX = "..SHARDTRACKER_CONFIG.SHARDBUTTONX);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SHARDBUTTONY = "..SHARDTRACKER_CONFIG.SHARDBUTTONY);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.HEALTHBUTTONX = "..SHARDTRACKER_CONFIG.HEALTHBUTTONX);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.HEALTHBUTTONY = "..SHARDTRACKER_CONFIG.HEALTHBUTTONY);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SOULBUTTONX = "..SHARDTRACKER_CONFIG.SOULBUTTONX);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SOULBUTTONY = "..SHARDTRACKER_CONFIG.SOULBUTTONY);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SPELLBUTTONX = "..SHARDTRACKER_CONFIG.SPELLBUTTONX);
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SPELLBUTTONY = "..SHARDTRACKER_CONFIG.SPELLBUTTONY);  

    ShardTrackerDebug("Centers:");
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SHARDBUTTON = "..ShardTrackerMinimapButton:GetCenter());
    ShardTrackerDebug("SHARDTRACKER_CONFIG.HEALTHBUTTON = "..ShardTrackerMinimapButtonHealth:GetCenter());
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SOULBUTTON = "..ShardTrackerMinimapButtonSoul:GetCenter());
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SPELLBUTTON = "..ShardTrackerMinimapButtonSpell:GetCenter());

    ShardTrackerDebug("Visibility:");
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SHARDBUTTON visible = "..tostring(ShardTrackerMinimapButton:IsVisible()));
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SHARDBUTTON shown = "..tostring(ShardTrackerMinimapButton:IsShown()));
    ShardTrackerDebug("SHARDTRACKER_CONFIG.SHARDBUTTON enabled = "..tostring(ShardTrackerMinimapButton:IsEnabled()));
end

---------------------------------------------------------------------------------
--
-- Show the tooltip text
-- 
---------------------------------------------------------------------------------
function ShardTracker_ShowShardButtonTooltip(theText1, theText2, theText3, theText4, theText5)

    -- Set the anchor and text for the tooltip.
    ShardTracker_MoneyToggle();
    Sea.wow.tooltip.clear("GameTooltip");
    ShardTracker_MoneyToggle();

    local x,y = this:GetCenter();
    if (x > GetScreenWidth()/2) then
        ShardTracker_MoneyToggle();    
        GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT");
        ShardTracker_MoneyToggle();
    else
        ShardTracker_MoneyToggle();    
        GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
        ShardTracker_MoneyToggle();
    end
    
    ShardTracker_MoneyToggle();
    GameTooltip:SetText(theText1, 0.39, 0.77, 0.16);
    ShardTracker_MoneyToggle();
    
    -- if we're not a warlock and this is a healthstone, show a different tooltip
    if (UnitClass("player") ~= "Warlock" and theText1 == SHARDTRACKER_HEALTHBUTTON_TIP1) then
        ShardTracker_MoneyToggle();
        GameTooltip:AddLine(SHARDTRACKER_HEALTHBUTTON_TIP5, 0.82, 0.24, 0.79);
        ShardTracker_MoneyToggle();
    else
    
        -- add description lines to the tooltip
        if (theText2) then
            ShardTracker_MoneyToggle();
            GameTooltip:AddLine(theText2, 0.82, 0.24, 0.79);
            ShardTracker_MoneyToggle();
        end
        if (theText3) then
            ShardTracker_MoneyToggle();
            GameTooltip:AddLine(theText3, 0.82, 0.24, 0.79);
            ShardTracker_MoneyToggle();
        end
        if (theText4) then
            ShardTracker_MoneyToggle();
            GameTooltip:AddLine(theText4, 0.82, 0.24, 0.79);
            ShardTracker_MoneyToggle();
        end
        if (theText5) then
            ShardTracker_MoneyToggle();
            GameTooltip:AddLine(theText5, 0.82, 0.24, 0.79);
            ShardTracker_MoneyToggle();
        end
    end
    
    -- Adjust width and height to account for new lines and show the tooltip
    -- (the Show() command automatically adjusts the width/height)
    GameTooltip:Show();

end


-----------------------------------------------------------------------------------
--
-- Set the scale of the shardtracker buttons
--
-----------------------------------------------------------------------------------
function ShardTracker_SetButtonScale(scaleName, slashCmd)
    local theScale;
    theScale = 0;
    
    if (scaleName == "" or scaleName == SHARDTRACKER_SCALE_1_CMD) then
        theScale = SHARDTRACKER_SCALE_1;
        SHARDTRACKER_CONFIG.BUTTONSCALE = SHARDTRACKER_SCALE_1_CMD;
        if (slashCmd) then ShardTrackerPrint("ShardTracker button scale set to "..SHARDTRACKER_SCALE_1_CMD.."."); end
    elseif (scaleName == SHARDTRACKER_SCALE_2_CMD) then
        theScale = SHARDTRACKER_SCALE_2;
        SHARDTRACKER_CONFIG.BUTTONSCALE = SHARDTRACKER_SCALE_2_CMD;
        if (slashCmd) then ShardTrackerPrint("ShardTracker button scale set to "..SHARDTRACKER_SCALE_2_CMD.."."); end
    elseif (scaleName == SHARDTRACKER_SCALE_3_CMD) then
        theScale = SHARDTRACKER_SCALE_3;
        SHARDTRACKER_CONFIG.BUTTONSCALE = SHARDTRACKER_SCALE_3_CMD;
        if (slashCmd) then ShardTrackerPrint("ShardTracker button scale set to "..SHARDTRACKER_SCALE_3_CMD.."."); end
    elseif (scaleName == SHARDTRACKER_SCALE_4_CMD) then
        theScale = SHARDTRACKER_SCALE_4;
        SHARDTRACKER_CONFIG.BUTTONSCALE = SHARDTRACKER_SCALE_4_CMD;
        if (slashCmd) then ShardTrackerPrint("ShardTracker button scale set to "..SHARDTRACKER_SCALE_4_CMD.."."); end
    elseif (scaleName == "debug") then
        ShardTrackerDebug("SHARDTRACKER_SCALE_1 = "..SHARDTRACKER_SCALE_1.."Minimap:GetScale() = "..Minimap:GetScale().." ShardTracker_ShardButton:GetScale() = "..ShardTracker_ShardButton:GetScale());
    else
        ShardTrackerPrint("Usage: /shardtracker scale "..SHARDTRACKER_SCALE_1_CMD.."|"..SHARDTRACKER_SCALE_2_CMD.."|"..SHARDTRACKER_SCALE_3_CMD.."|"..SHARDTRACKER_SCALE_4_CMD);        
    end
    
    if (theScale ~= 0) then
        ShardTrackerDebug("Scale set to: "..theScale);

        -- set the scales
        ShardTracker_ShardButton:SetScale(theScale);
        ShardTracker_HealthStoneButton:SetScale(theScale);
        ShardTracker_SoulStoneButton:SetScale(theScale);
        ShardTracker_SpellStoneButton:SetScale(theScale);
        
        ShardTracker_ButtonSetLocation();
    end
end


-----------------------------------------------------------------------------------
--
-- Initialize the scale of the shardtracker buttons
--
-----------------------------------------------------------------------------------
function ShardTracker_InitButtonScales()

    ShardTrackerTron("ShardTracker_InitButtonScales");
    
    -- determine the scales based on our resolution
    SHARDTRACKER_BASESCALE = Minimap:GetScale();
    SHARDTRACKER_SCALE_1  = SHARDTRACKER_BASESCALE
    SHARDTRACKER_SCALE_2  = SHARDTRACKER_BASESCALE * 1.25;
    SHARDTRACKER_SCALE_3  = SHARDTRACKER_BASESCALE * 1.5;
    SHARDTRACKER_SCALE_4  = SHARDTRACKER_BASESCALE * 1.75;

    ShardTrackerDebug("SHARDTRACKER_SCALE_1 = "..SHARDTRACKER_SCALE_1.."Minimap:GetScale() = "..Minimap:GetScale().." ShardTracker_ShardButton:GetScale() = "..ShardTracker_ShardButton:GetScale());
    
    -- set the button scales    
    ShardTracker_SetButtonScale(SHARDTRACKER_CONFIG.BUTTONSCALE);
end


-----------------------------------------------------------------------------------
--
-- Show the "soulstone ready" alerts
--
-----------------------------------------------------------------------------------
function ShardTracker_NotifySoulstoneReady()
    if (SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP == 1 or SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP == true) then
        ShardTrackerSortText:SetText(SHARDTRACKER_SOULSTONEREADYMSG);
        ShardTrackerSortText:SetTextColor(1.0,0.0,0.0);
        ShardTrackerSortFrame:SetAlpha(1.0);
        ShardTrackerSortFrame:Show();
        UIFrameFadeOut(ShardTrackerSortFrame, 5);
    end
    ShardTracker_PlaySound("AuctionWindowClose");
    ShardTrackerPrint("Soulstone Resurrection is ready to recast!",1.0,0.0,0.0);
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              UTILITY FUNCTIONS                                             --
--                                                                                            --
--============================================================================================--
--============================================================================================--


---------------------------------------------------------------------------------
--
-- Put an item into a free bag slot
-- 
---------------------------------------------------------------------------------
function ShardTracker_BagItem()

    -- for each bag and slot
    for theBag = 0, 4, 1 do
        local numSlot = GetContainerNumSlots(theBag);
        for theSlot = 1, numSlot, 1 do
        
            -- get info about the item here
            local texture, itemCount, locked = GetContainerItemInfo(theBag, theSlot);
            
            -- if we found nothing, add us to the list of free bag slots
            if (not texture) then
                PickupContainerItem(theBag,theSlot);
                return true;
            end
        end
    end
    
    AutoEquipCursorItem();
    
    return false;
end


---------------------------------------------------------------------------------
--
-- Return true if the item in the player's main hand is two handed
-- 
---------------------------------------------------------------------------------
function ShardTracker_IsWeaponTwoHanded()

    -- get the id number for the item slot
    local id = GetInventorySlotInfo("MainHandSlot");

    -- point the tooltip at this slot
    ShardTracker_MoneyToggle();
    local hasItem = ShardTrackerTooltip:SetInventoryItem("player", id);
    ShardTracker_MoneyToggle();
    if (hasItem) then
        local itemName1 = "";    
        ShardTracker_MoneyToggle();
        ttext = getglobal("ShardTrackerTooltipTextLeft2");
        if (ttext and ttext:IsVisible() and ttext:GetText() ~= nil) then
            itemName1 = ttext:GetText();
        end
        ShardTracker_MoneyToggle();
        local itemName2 = "";    
        ShardTracker_MoneyToggle();
        ttext = getglobal("ShardTrackerTooltipTextLeft3");
        if (ttext and ttext:IsVisible() and ttext:GetText() ~= nil) then
            itemName2 = ttext:GetText();
        end
        ShardTracker_MoneyToggle();

        if (itemName1 == "Two-Hand" or itemName2 == "Two-Hand") then
            return true;
        end
    end
    
    return false;
end

-----------------------------------------------------------------------------------
--
-- Return the name of the item in the specified bag and slot
--
-----------------------------------------------------------------------------------
function ShardTracker_GetItemNameInBagSlot(theBag, theSlot, clearToolTip)

    local itemName = nil;
    
    -- point our tooltip at this item so we can read its name, etc
    ShardTracker_MoneyToggle();
    getglobal("ShardTrackerTooltip"):SetBagItem(theBag, theSlot);
    local ttext = getglobal("ShardTrackerTooltipTextLeft1");
    ShardTracker_MoneyToggle();

    -- if the tooltip had info, store it
    if (ttext and ttext:IsVisible() and ttext:GetText() ~= nil) then
        itemName = ttext:GetText();
    end
    
    if (clearToolTip) then
        ShardTracker_MoneyToggle();
        Sea.wow.tooltip.clear("ShardTrackerTooltip");
        ShardTracker_MoneyToggle();
    end
    return itemName;
end


-----------------------------------------------------------------------------------
--
-- Return the name of the item in the specified tooltip text
--
-----------------------------------------------------------------------------------
function ShardTracker_GetItemName(itemTip)
    if (type(itemTip) ~= "table") then
        return itemTip;
    elseif (type(itemTip[1]) == "table") then
        return itemTip[1]["left"];
    else
        return "NIL";
    end
end


-----------------------------------------------------------------------------------
--
-- Return the bag, slot location of the specified item
--
-----------------------------------------------------------------------------------
function ShardTrackerFindItemBagSlotByName(theItemName)
    
    -- for each bag
    for bag = 4, 0, -1 do
        local size = GetContainerNumSlots(bag);
        if (size > 0) then
        
            -- for each slot in the bag
            for slot=1, size, 1 do
            
                -- get info about the item in this slot
                local texture, itemCount = GetContainerItemInfo(bag, slot);
                if (itemCount) then
                    local itemName = ShardTracker_GetItemNameInBagSlot(bag, slot, false);
                                            
                    -- if the item has a name
                    if (itemName ~= "") then

                        -- if the item is a soul shard, increase the count
                        if (itemName == theItemName) then
                            return bag, slot;
                        end
                    end
                end
            end
        end
    end
    
    return nil, nil;
end


-----------------------------------------------------------------------------------
--
-- Given a player's name, return their party member number
--
-----------------------------------------------------------------------------------
function ShardTracker_GetPartyMemberNumber(theName)
    local numberOfPartyMembers = GetNumPartyMembers();
    for i = 1, numberOfPartyMembers do
        if (UnitName("party"..i) == theName) then
            return i
        end
    end
    return 0
end


-----------------------------------------------------------------------------------
--
-- Create a list of spell IDs for spells we might want to cast (such as Create Healthstone)
--
-----------------------------------------------------------------------------------
function ShardTracker_BuildSpellTable()

    ShardTracker_CreateSoulStoneSpellID   = nil;
    ShardTracker_CreateHealthStoneSpellID = nil;
    ShardTracker_CreateSpellStoneSpellID = nil;
    
    -- as we look through the spellbook, these variables will
    -- allow us to remember the highest level of a spell that
    -- we're found overall, since we only want to case the
    -- highest level create health/soulstone spell
    local HighestSoulstoneSpellLevelSeen = 0;
    local HighestHealthstoneSpellLevelSeen = 0;
    local HighestSpellstoneSpellLevelSeen = 0;
    local spellLevel = 0;
    
    -- for all 180 possible spell entries
    for id = 1, 180 do 
        local spellName, subSpellName = GetSpellName(id, "spell");
        if (spellName) then
        
            -- find the highest level healthstone we can make
            if (string.find(spellName, "Healthstone", 1, true)) then
                spellLevel = 0;
                if (spellName == "Create Healthstone (Minor)") then
                    spellLevel = 1;
                elseif (spellName == "Create Healthstone (Lesser)") then
                    spellLevel = 2;
                elseif (spellName == "Create Healthstone") then
                    spellLevel = 3;
                elseif (spellName == "Create Healthstone (Greater)") then
                    spellLevel = 4;
                elseif (spellName == "Create Healthstone (Major)") then
                    spellLevel = 5;
                end
                if (spellLevel > HighestHealthstoneSpellLevelSeen) then
                    ShardTracker_CreateHealthStoneSpellID = id;
                    HighestHealthstoneSpellLevelSeen = spellLevel;
                end
                
            -- find the highest level soulstone we can make
            elseif (string.find(spellName, "Soulstone", 1, true)) then
                spellLevel = 0;
                if (spellName == "Create Soulstone (Minor)") then
                    spellLevel = 1;
                elseif (spellName == "Create Soulstone (Lesser)") then
                    spellLevel = 2;
                elseif (spellName == "Create Soulstone") then
                    spellLevel = 3;
                elseif (spellName == "Create Soulstone (Greater)") then
                    spellLevel = 4;
                elseif (spellName == "Create Soulstone (Major)") then
                    spellLevel = 5;
                end
                if (spellLevel > HighestSoulstoneSpellLevelSeen) then
                    ShardTracker_CreateSoulStoneSpellID = id;
                    HighestSoulstoneSpellLevelSeen = spellLevel;
                end
                
            -- find the highest level spellstone we can make
            elseif (string.find(spellName, "Spellstone", 1, true)) then
                spellLevel = 0;
                if (spellName == "Create Spellstone") then
                    spellLevel = 3;
                elseif (spellName == "Create Spellstone (Greater)") then
                    spellLevel = 4;
                elseif (spellName == "Create Spellstone (Major)") then
                    spellLevel = 5;
                end
                if (spellLevel > HighestSpellstoneSpellLevelSeen) then
                    ShardTracker_CreateSpellStoneSpellID = id;
                    HighestSpellstoneSpellLevelSeen = spellLevel;
                end
            end
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Our own print function
--
-----------------------------------------------------------------------------------
function ShardTrackerPrint(theMsg, r, g, b)
    
    -- 0.73, 0.53, 0.87
    if (r == nil) then r = 0.73; end
    if (g == nil) then g = 0.53; end
    if (b == nil) then b = 0.87; end

    Print(theMsg, r, g, b);
end


-----------------------------------------------------------------------------------
--
-- Show the info text
--
-----------------------------------------------------------------------------------
function ShardTracker_ShowInfoMessage()
    DEFAULT_CHAT_FRAME:AddMessage(" ");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."ShardTracker is a Soul Shard management tool for Warlocks.", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."It has two primary functions: keeping you informed about", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."your own shards, and keeping you informed about your party ", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."members' Healthstone status.  In the first function, your", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."own shards are managed via four small buttons.  These are", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."initially grouped near your radar, but you may position", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."them where ever you like using the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_LOCK_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." and "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET.."unlock"..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." functions.", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."(Note: grabbing onto the buttons to drag can be hit and miss.)", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."You can also scale the buttons using the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_SCALE_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.  Four");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."scales are available: regular, large, biggie, and supersizeme.");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."You may toggle the background image of the Shard button using");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_SHARDBG_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.  Finally, you can "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_RESET_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." the positions of");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."the buttons to their original locations, or if other objects");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."are blocking the original locations, you can use the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_CENTER_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."command to center them onscreen for your placement.");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_LTGREEN.."The Soul Shard button"..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." lets you know how many shards you", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."have in your inventory.  This button will turn red and flash", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."when the number of shards drops below a certain level.  Currently", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."this level is set to "..SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT..", but you may set it to any number", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."you like using the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_LIMIT_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.  Clicking on this button will", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."sort your shards (including Healthstone, Soulstone, and", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."Spellstones) into the pack of your choice.  Right now, this", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."is set to pack number "..SHARDTRACKER_CONFIG.THE_SORT_BAG..", but you may change this using the", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_BAG_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.  "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_LTGREEN.."The Healthstone button"..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." lets you know if you", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."have a Healthstone.  If you don't have one, clicking", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."on this button will create your highest rank Healthstone.", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."Clicking again will use your Healthstone.  Additionally,", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."if you have a friendly player selected, clicking will place", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."the Healthstone into a trade window with that player.  "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_LTGREEN.."The", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_LTGREEN.."Soulstone button"..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." informs you about the status of your Soulstone.", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."When you have no Soulstone, the button will be greyed.  Clicking", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."on this button will create your highest rank Soulstone.", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."Clicking again will cast Soulstone Resurrection on any friendly", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."target you have selected.  Once cast, the Soulstone button will", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."show a counter representing the number of minutes until you can", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."cast your next Soulstone Resurrection spell.  When this count-", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."down ends, an alarm will sound and the button will flash to", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."remind you to re-cast Soulstone Resurrection.  You can optionally", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."disable the audible alarm using the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_SOUND_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.  To disable",nil,nil,nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."the popup notification, use the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_SOUL_POPUP_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.  "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_LTGREEN.."The Spellstone", nil, nil, nil);    
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_LTGREEN.."button"..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." shows your Spellstone status.  Clicking this button will", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."create your highest rank Spellstone.  Clicking again will equip", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."your spellstone and display the number of seconds until you can", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."use your Spellstone.  Once the timer ends, clicking will activate", nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."your Spellstone as well as automatically re-equip whatever weapon");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."you were wielding before you equipped the Spellstone.");
    DEFAULT_CHAT_FRAME:AddMessage(" ");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."The second functionally of the AddOn involves notifying you of your");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."party members' Healthstone status.  Whenever a party member gains a");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."Healthstone, an icon will appear over her party portrait.  When");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."a party member uses her Healthstone, this icon will flash and");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."an alarm will sound.  For this functionality to work, party members");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."must also be running the Shardtracker AddOn.  You may optionally");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."disable this functionality using the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_MONITOR_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." command.   In");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."addition, you may create a list of players to monitor.  Only those");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."players in your list will receive communication from ShardTracker.");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."You can "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_ADD_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." or "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_REMOVE_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." players from this list, and use the "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET..SHARDTRACKER_RESTRICT_CMD..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."command to enable or disable the list.  Players running Cosmos");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."that aren't on your list will still have their Healthstones");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."monitored, but ShardTracker will do so silently.");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_GREEN.."For specific commands, please see /shardtracker help.");
    ShardTracker_AddHelpMessage("");

end


-----------------------------------------------------------------------------------
--
-- Show the help text
--
-----------------------------------------------------------------------------------
function ShardTracker_ShowHelp()

    if (ShardTracker_DragLock) then
        lockedText = "Locked";
    else
        lockedText = "Unlocked";
    end
    
    local warlock;
    if (UnitClass("player") == "Warlock") then
        warlock = true;
    else
        warlock = false;
    end
    
    DEFAULT_CHAT_FRAME:AddMessage(" ");
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.."ShardTracker, an AddOn by Cragganmore.  "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET.."Version "..SHARDTRACKER_VERSION, nil, nil, nil);
    ShardTracker_AddHelpMessage(SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.."Based on code by Kithador and Ryu", nil, nil, nil);
    DEFAULT_CHAT_FRAME:AddMessage(SHARDTRACKER_COLOR_GREEN.."Usage: "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET.."/shardtracker <command>"..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN.." or "..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_VIOLET.."/st <command>"..SHARDTRACKER_COLOR_CLOSE);
    ShardTracker_AddHelpMessage("  "..SHARDTRACKER_ON_CMD.."|"..SHARDTRACKER_OFF_CMD, "Turns ShardTracker on or off.", nil, SHARDTRACKER_CONFIG.ENABLED);
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_TOGGLE_CMD.." [shards | healthstone | soulstone | spellstone]", "Toggles the respective button on or off.", nil, nil); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_MONITOR_CMD.." on|off", "Toggles whether you monitor your party's Healthstones.", nil, SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_BAG_CMD.." [0-4]", "Sets the default bag to sort shards (0 = backpack, 4 = leftmost bag).", SHARDTRACKER_CONFIG.THE_SORT_BAG, true); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_SORT_CMD, "Sorts your shards into the default bag (set with the \"bag\" command).", nil, nil); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_LIMIT_CMD.." [0-20]", "When your total Shards drops below this number, the shard button will flash to warn you.", SHARDTRACKER_CONFIG.MIN_FLASHING_SHARD_LIMIT, true); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_SOUND_CMD.." on|off", "Turns audible warnings on or off.", nil, SHARDTRACKER_CONFIG.USE_SOUND); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_SOUL_POPUP_CMD.." on|off", "Turns the Soulstone popup alert on or off.", nil, SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_SHARDBG_CMD.." on|off", "Turns the shard button background image on or off.", nil, SHARDTRACKER_CONFIG.SHOW_SHARD_BG); end
    ShardTracker_AddHelpMessage("  "..SHARDTRACKER_FLASH_CMD.." on|off", "Toggles whether your Healthstone button flashes when you need a new one.", nil, SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE);
    ShardTracker_AddHelpMessage("  "..SHARDTRACKER_UNLOCK_CMD, "When unlocked, the ShardTracker buttons are disabled and may be positioned anywhere on screen.", lockedText, true);
    ShardTracker_AddHelpMessage("  "..SHARDTRACKER_LOCK_CMD, "When locked, the ShardTracker buttons are enabled and unmovable.", lockedText, true);
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_RESET_CMD, "Resets the ShardTracker buttons to their default locations.", nil, nil); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_CENTER_CMD, "Places the ShardTracker buttons in the center of the screen, in case they're inaccessible due to other buttons near the radar.", nil, nil); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_SCALE_CMD.." "..SHARDTRACKER_SCALE_1_CMD.."|"..SHARDTRACKER_SCALE_2_CMD.."|"..SHARDTRACKER_SCALE_3_CMD.."|"..SHARDTRACKER_SCALE_4_CMD, "Sets the size/scale of the ShardTracker buttons.", SHARDTRACKER_CONFIG.BUTTONSCALE, true); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_RESTRICT_CMD.." on|off", "When enabled, ShardTracker will only communicate with party members on our \"OK to communicate with\" list.  Note: Party members running Cosmos will always be communicated with, but silently.", nil, SHARDTRACKER_CONFIG.LIST_RESTRICT); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_ADD_CMD, "Add a player to our \"OK to communicate with\" list.", nil, nil); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_REMOVE_CMD, "Remove a player from our \"OK to communicate with\" list.", nil, nil); end
    if (warlock) then ShardTracker_AddHelpMessage("  "..SHARDTRACKER_LIST_CMD, "Show our \"OK to communicate with\" list.", nil, nil); end

    ShardTracker_AddHelpMessage("  "..SHARDTRACKER_INFO_CMD, "Displays a detailed description of ShardTracker.", nil, nil);
    ShardTracker_AddHelpMessage("  "..SHARDTRACKER_HELP_CMD, "Displays this message.", nil, nil);
    DEFAULT_CHAT_FRAME:AddMessage(" ");
    ShardTracker_AddHelpMessage("Example: \"/st flash off\" turns off Healthstone flashing.", nil, nil, nil);
end

-----------------------------------------------------------------------------------
--
-- Prints a formatted help message
--
-----------------------------------------------------------------------------------
function ShardTracker_AddHelpMessage(command, detail, status, enabled)

    local message = "";
    
    if (detail == nil) then
        message = SHARDTRACKER_COLOR_VIOLET..command..SHARDTRACKER_COLOR_CLOSE;
    else
        message = SHARDTRACKER_COLOR_VIOLET..command..SHARDTRACKER_COLOR_CLOSE..SHARDTRACKER_COLOR_GREEN..": "..detail..SHARDTRACKER_COLOR_CLOSE;
    end
    
    if (enabled == nil) then
        DEFAULT_CHAT_FRAME:AddMessage(message);
        return;
    end
    
    if (status == "" or status == nil) then
        if (enabled and enabled ~= 0) then
            DEFAULT_CHAT_FRAME:AddMessage(message..SHARDTRACKER_COLOR_VIOLET.." (enabled)"..SHARDTRACKER_COLOR_CLOSE);
        else
            DEFAULT_CHAT_FRAME:AddMessage(message..SHARDTRACKER_COLOR_GREY.." (disabled)"..SHARDTRACKER_COLOR_CLOSE);
        end
    else
        if (enabled and enabled ~= 0) then
            DEFAULT_CHAT_FRAME:AddMessage(message..SHARDTRACKER_COLOR_VIOLET.." (set to "..status..")"..SHARDTRACKER_COLOR_CLOSE);
        else
            DEFAULT_CHAT_FRAME:AddMessage(message..SHARDTRACKER_COLOR_CLOSE.." (set to "..status..")"..SHARDTRACKER_COLOR_CLOSE);
        end     
    end
end




--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              TOGGLING FUNCTIONS                                    --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Toggle on or off flashing healthstones
--
-----------------------------------------------------------------------------------
function ShardTracker_Toggle_Flash_Health(toggle, slashCmd)
    if (not toggle) then
        if (SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE == 1) then
            toggle = 0;
        else
            toggle = 1;
        end
    end
    if (toggle == 1 and SHARDTRACKER_CONFIG.ENABLED == 1) then
        SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE = 1;
        if (slashCmd == true) then ShardTrackerPrint("ShardTracker: Healthstone button will flash when you need a new one."); end
        if (UnitClass("player") ~= "Warlock") then
            ShardTracker_HealthStoneButton:Show();
        end
    else
        SHARDTRACKER_CONFIG.FLASH_HEALTHSTONE = 0;
        if (slashCmd == true) then ShardTrackerPrint("ShardTracker: Healthstone button will no longer flash."); end
        ShardTracker_HealthStoneButton:SetAlpha(1.0);
        if (UnitClass("player") ~= "Warlock") then
            ShardTracker_HealthStoneButton:Hide();
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Handle the toggling on and off of various aspects of the plugin
--
-----------------------------------------------------------------------------------
function ShardTracker_Toggle_Shards(toggle)
    if (toggle == nil) then
        if (SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED == 1) then
            toggle = 0;
        else
            toggle = 1;
        end
    end
    if (toggle == 1 and UnitClass("player") == "Warlock" and SHARDTRACKER_CONFIG.ENABLED == 1) then
        SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED = 1;
        ShardTracker_ShardButton:Show();
        ShardTracker_ShardCountLabel:Show();
    else
        SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED = 0;
        ShardTracker_ShardButton:Hide();
        ShardTracker_ShardCountLabel:Hide();
        ShardTracker_ShardButton:SetAlpha(1.0);
        ShardTracker_ShardCountLabel:SetAlpha(1.0);
    end
end
function ShardTracker_Toggle_Health(toggle)
    if (toggle == nil) then
        if (SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED == 1) then
            toggle = 0;
        else
            toggle = 1;
        end
    end
    if (toggle == 1 and SHARDTRACKER_CONFIG.ENABLED == 1) then
        SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED = 1;
        ShardTracker_HealthStoneButton:Show();
    else
        SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED = 0;
        ShardTracker_HealthStoneButton:Hide();
    end
end
function ShardTracker_Toggle_Soul(toggle)
    if (toggle == nil) then
        if (SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED == 1) then
            toggle = 0;
        else
            toggle = 1;
        end
    end
    if (toggle == 1 and UnitClass("player") == "Warlock" and SHARDTRACKER_CONFIG.ENABLED == 1) then
        SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED = 1;
        --if (ShardTracker_HaveSoulStone == 1) then
            ShardTracker_SoulStoneButton:Show();
            ShardTracker_SoulStoneText:Show();
        --end
    else
        SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED = 0;
        ShardTracker_SoulStoneButton:Hide();
        ShardTracker_SoulStoneButton:SetAlpha(1.0);
        ShardTracker_SoulStoneText:Hide();
        ShardTracker_SoulStoneText:SetAlpha(1.0);
    end
end
function ShardTracker_Toggle_Spell(toggle)
    if (toggle == nil) then
        if (SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED == 1) then
            toggle = 0;
        else
            toggle = 1;
        end
    end
    if (toggle == 1 and UnitClass("player") == "Warlock" and SHARDTRACKER_CONFIG.ENABLED == 1) then
        SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED = 1;
        ShardTracker_SpellStoneButton:Show();
        ShardTracker_SpellStoneText:Show();
    else
        SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED = 0;
        ShardTracker_SpellStoneButton:Hide();
        ShardTracker_SpellStoneButton:SetAlpha(1.0);
        ShardTracker_SpellStoneText:Hide();
        ShardTracker_SpellStoneText:SetAlpha(1.0);
    end
end
function ShardTracker_Toggle(toggle)
    if (toggle == nil) then
        if (SHARDTRACKER_CONFIG.ENABLED == 1 and not SHARDTRACKER_FIRST_TIME.ENABLED) then
            toggle = 0;
        else
            toggle = 1;
        end
    end
    if (toggle == 1) then
        if (not SHARDTRACKER_FIRST_TIME.ENABLED and (SHARDTRACKER_CONFIG.ENABLED ~= toggle)) then
            ShardTrackerPrint("ShardTracker: Enabled");
        else
            SHARDTRACKER_FIRST_TIME.ENABLED = false;
        end
    else
        if (SHARDTRACKER_CONFIG.ENABLED ~= toggle) then
            ShardTrackerPrint("ShardTracker: Disabled");
        end
    end
    
    SHARDTRACKER_CONFIG.ENABLED = toggle;
    
    ShardTracker_Toggle_Shards(toggle);
    ShardTracker_Toggle_Health(toggle);
    ShardTracker_Toggle_Soul(toggle);   
    ShardTracker_Toggle_Spell(toggle);
    
    ShardTrackerDebug("Toggle: SHARDTRACKER_CONFIG.ENABLED = "..SHARDTRACKER_CONFIG.ENABLED);
    ShardTrackerDebug("Toggle: SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED = "..SHARDTRACKER_CONFIG.SHARDBUTTON_ENABLED);
    ShardTrackerDebug("Toggle: SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED = "..SHARDTRACKER_CONFIG.HEALTHBUTTON_ENABLED);
    ShardTrackerDebug("Toggle: SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED = "..SHARDTRACKER_CONFIG.SOULBUTTON_ENABLED);
    ShardTrackerDebug("Toggle: SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED = "..SHARDTRACKER_CONFIG.SPELLBUTTON_ENABLED);

end

-----------------------------------------------------------------------------------
--
-- Process the toggle slash command
--
-----------------------------------------------------------------------------------
function ShardTracker_ToggleSlashCommand(var)
    if (var == "shards") then
        ShardTracker_Toggle_Shards();
    elseif (var == "healthstone") then
        ShardTracker_Toggle_Health();
    elseif (var == "soulstone") then
        ShardTracker_Toggle_Soul();   
    elseif (var == "spellstone") then
        ShardTracker_Toggle_Spell();
    elseif (var == "") then
        ShardTracker_Toggle();
    else
        ShardTrackerPrint("Usage: /shardtracker toggle \[shards\|healthstone\|soulstone\|spellstone\]");
    end
end

-----------------------------------------------------------------------------------
--
-- Toggle Debug Messages
--
-----------------------------------------------------------------------------------
function ShardTracker_ToggleDebug()
    SHARDTRACKER_DEBUG = (not SHARDTRACKER_DEBUG);
    if (SHARDTRACKER_DEBUG) then
        ShardTrackerPrint("ShardTracker: Debug On");
    else
        ShardTrackerPrint("ShardTracker: Debug Off");
    end
end


function ShardTracker_PrintRiddle()
    if (message) then
        message("What's tall as a house,\nRound as a cup,\nAnd all the king's horses\nCan't draw it up?");
    else
        Print("What's tall as a house,\nRound as a cup,\nAnd all the king's horses\nCan't draw it up?", 1.0, 1.0, 0.5);
    end
end


---------------------------------------------------------------------------------
--
-- Toggle whether to play sounds when soulstone cooldowns expire / players need new healthstones
-- 
---------------------------------------------------------------------------------
function ShardTracker_Toggle_Sound(toggle)
    if (not toggle) then
        if (SHARDTRACKER_CONFIG.USE_SOUND == true) then
            toggle = false;
        else
            toggle = true;
        end
    end
    if (toggle == 0) then toggle = false end;
    if (toggle == 1) then toggle = true end;
    if (toggle) then
        if (not SHARDTRACKER_FIRST_TIME.SOUND and (SHARDTRACKER_CONFIG.USE_SOUND ~= toggle)) then
            ShardTrackerPrint("ShardTracker: Sound On");
        else
            SHARDTRACKER_FIRST_TIME.SOUND = false;
        end
        SHARDTRACKER_CONFIG.USE_SOUND = true;
    else
        if (SHARDTRACKER_CONFIG.USE_SOUND ~= toggle) then
            ShardTrackerPrint("ShardTracker: Sound Off");
        end
        SHARDTRACKER_CONFIG.USE_SOUND = false;
    end
end



---------------------------------------------------------------------------------
--
-- Toggle whether to play sounds when soulstone cooldowns expire / players need new healthstones
-- 
---------------------------------------------------------------------------------
function ShardTracker_Toggle_SoulPop(toggle)
    if (not toggle) then
        if (SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP == true) then
            toggle = false;
        else
            toggle = true;
        end
    end
    if (toggle == 0) then toggle = false end;
    if (toggle == 1) then toggle = true end;
    if (toggle) then
        if (not SHARDTRACKER_FIRST_TIME.SOULPOP and (SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP ~= toggle)) then
            ShardTrackerPrint("ShardTracker: Soulstone Popup Message Enabled.");
        else
            SHARDTRACKER_FIRST_TIME.SOULPOP = false;
        end
        SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP = true;
    else
        if (SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP ~= toggle) then
            ShardTrackerPrint("ShardTracker: Soulstone Popup Message Disabled.");
        end
        SHARDTRACKER_CONFIG.SHOW_SOUL_POPUP = false;
    end
end

---------------------------------------------------------------------------------
--
-- Toggle whether to only send healthstone monitoring messages to people on our
-- "OK to send" list
-- 
---------------------------------------------------------------------------------
function ShardTracker_Toggle_RestrictedChat(toggle, slashCmd)
    if (not toggle) then
        if (SHARDTRACKER_CONFIG.LIST_RESTRICT == true) then
            toggle = false;
        else
            toggle = true;
        end
    end
    if (toggle == 0) then toggle = false end;
    if (toggle == 1) then toggle = true end;
    if (toggle) then
        if ((slashCmd) or (not SHARDTRACKER_FIRST_TIME.LIST_RESTRICT and (SHARDTRACKER_CONFIG.LIST_RESTRICT ~= toggle))) then
            ShardTrackerPrint("ShardTracker will only communicate with party members on our communication list.");
        else
            SHARDTRACKER_FIRST_TIME.LIST_RESTRICT = false;
        end
        ShardTracker_InitializePartyHealthstoneIcons();
        ShardTracker_ReQueryAllPartyMembers();
        SHARDTRACKER_CONFIG.LIST_RESTRICT = true;
    else
        if (SHARDTRACKER_CONFIG.LIST_RESTRICT ~= toggle) then
            ShardTrackerPrint("ShardTracker will attempt to communicate with all party members.");
        end
        ShardTracker_InitializePartyHealthstoneIcons();
        ShardTracker_ReQueryAllPartyMembers();
        SHARDTRACKER_CONFIG.LIST_RESTRICT = false;
    end
end


---------------------------------------------------------------------------------
--
-- Toggle the shard background image
-- 
---------------------------------------------------------------------------------
function ShardTracker_Toggle_ShardBG(toggle)
    if (not toggle) then
        if (SHARDTRACKER_CONFIG.SHOW_SHARD_BG == true) then
            toggle = false;
        else
            toggle = true;
        end
    end
    if (toggle == 0) then toggle = false end;
    if (toggle == 1) then toggle = true end;
    if (toggle) then
        if (not SHARDTRACKER_FIRST_TIME.SHARDBG and (SHARDTRACKER_CONFIG.SHOW_SHARD_BG ~= toggle)) then
            ShardTrackerPrint("ShardTracker: Shard button background enabled.");
        else
            SHARDTRACKER_FIRST_TIME.SHARDBG = false;
        end
        SHARDTRACKER_CONFIG.SHOW_SHARD_BG = true;
    else
        if (SHARDTRACKER_CONFIG.SHOW_SHARD_BG ~= toggle) then
            ShardTrackerPrint("ShardTracker: Shard button background disabled.");
        end
        SHARDTRACKER_CONFIG.SHOW_SHARD_BG = false;
    end

    ShardTracker_UpdateShardButton(ShardTracker_TotalShards);    
end




---------------------------------------------------------------------------------
--
-- Toggle whether to monitor when your party members use / need healthstones
-- 
---------------------------------------------------------------------------------
function ShardTracker_Toggle_Monitor_Party_Healthstones(toggle, slashCommand)

    if (not toggle) then
        if (SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES == 1) then
            toggle = 0;
        else
            toggle = 1;
        end
    end

    -- always look for chat msgs if not a warlock.  dummyproof.
    if (UnitClass("player") ~= "Warlock") then 
        toggle = 1;  
        if (slashCommand) then
            ShardTrackerPrint("This function is only available to Warlocks.");
        end
    end

    if (toggle == 1) then
        if ((not SHARDTRACKER_FIRST_TIME.MONITOR or slashCommand) and (SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES ~= toggle)) then
            SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES = 1;
            if (slashCommand) then ShardTracker_ShowHealthstoneMonitorStatus(); end
            ShardTracker_InitializePartyHealthstoneIcons();
            ShardTracker_ReQueryAllPartyMembers();
            
            SHARDTRACKER_FIRST_TIME.MONITOR = false;
        else
            SHARDTRACKER_FIRST_TIME.MONITOR = false;
            SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES = 1;
        end
    else
        if (SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES ~= toggle) then
            SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES = 0;
            if (slashCommand) then ShardTracker_ShowHealthstoneMonitorStatus(); end
            ShardTracker_InitializePartyHealthstoneIcons();
            SHARDTRACKER_FIRST_TIME.MONITOR = false;
        end
        SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES = 0;
        SHARDTRACKER_FIRST_TIME.MONITOR = false;
    end
end


---------------------------------------------------------------------------------
--
-- Display a message letting the player know whether healthstone monitoring is on or off
-- 
---------------------------------------------------------------------------------
function ShardTracker_ShowHealthstoneMonitorStatus()
    if (UnitClass("player") == "Warlock") then
        if (SHARDTRACKER_CONFIG.MONITOR_PARTY_HEALTHSTONES == 1) then
            ShardTrackerPrint("ShardTracker is monitoring your party's healthstones.  For this to work, your party members must also be running ShardTracker.  To disable, use \"/st monitor off\".");
        else
            ShardTrackerPrint("ShardTracker is not monitoring your party's Healthstones.  To enable, use \"/st monitor on\" and be sure your party members are also running ShardTracker.");
        end
    end
end


---------------------------------------------------------------------------------
--
-- Play a sound
-- 
---------------------------------------------------------------------------------
function ShardTracker_PlaySound(theSoundName)
    if (SHARDTRACKER_CONFIG.USE_SOUND == true) then
        PlaySound(theSoundName);
    end
end




--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              DEBUGGING FUNCTIONS                                   --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Print a debug message
--
-----------------------------------------------------------------------------------
function ShardTrackerDebug(theMsg)
    if (SHARDTRACKER_DEBUG) then
        DEFAULT_CHAT_FRAME:AddMessage(theMsg, 1.0, 1.0, 0.2);
    end
end


-----------------------------------------------------------------------------------
--
-- Print a trace message
--
-----------------------------------------------------------------------------------
function ShardTrackerTron(theMsg)
    if (SHARDTRACKER_DEBUG or SHARDTRACKER_TRON) then
        DEFAULT_CHAT_FRAME:AddMessage(theMsg, 0.3, 0.6, 0.3);
    end
end




-----------------------------------------------------------------------------------
--
-- Dump a debug snapshot of the current party list
--
-----------------------------------------------------------------------------------
function ShardTracker_DebugDumpPartyList()

    ShardTrackerDebug("Dumping Party list with "..table.getn(ShardTracker_PartyMemberList).." members in the list:");
    for i = 1, table.getn(ShardTracker_PartyMemberList) do
        Print("     "..i.." = "..ShardTracker_PartyMemberList[i].name.." , hasHealthstone = "..tostring(ShardTracker_PartyMemberList[i].hasHealthstone).." , answeredPing = "..tostring(ShardTracker_PartyMemberList[i].answeredPing));
    end
end


-----------------------------------------------------------------------------------
--
-- Print a debug report of the current status of the plugin
--
-----------------------------------------------------------------------------------
function ShardTracker_DebugPrintStatus()

    -- show the party members' healthstone status
    ShardTrackerPrint("Summary of party member Healthstones based on ShardTracker_PartyMemberList ("..table.getn(ShardTracker_PartyMemberList).." members in list):");
    for i = 1, table.getn(ShardTracker_PartyMemberList) do
        ShardTrackerPrint("   Member #"..i.." is "..ShardTracker_PartyMemberList[i].name.." Healthstone = "..tostring(ShardTracker_PartyMemberList[i].hasHealthstone).." Flashing = "..tostring(ShardTracker_PartyMemberList[i].flashing));
    end
end


-----------------------------------------------------------------------------------
--
-- Code for debugging the party healthstone code
--
-----------------------------------------------------------------------------------
function ShardTracker_DebugHealthstonePartyCode(command, thePartyMember)

    if (var == "") then
        ShardTrackerPrint("ShardTracker_DebugHealthstonePartyCode: Usage - /st testyes/testno [charname]");
        return;
    end
    
    -- find this party member's number in the party
    senderNumber = ShardTracker_GetPartyMemberNumber(thePartyMember);

    if (senderNumber ~= 0) then
        if (command == "testyes") then
            ShardTrackerPrint("ShardTracker_DebugHealthstonePartyCode: Setting "..thePartyMember.." to have a healthstone!");
            ShardTracker_UpdatePartyHealthstoneList(thePartyMember, true, false);
        elseif (command == "testno") then
            ShardTrackerPrint("ShardTracker_DebugHealthstonePartyCode: Setting "..thePartyMember.." to need a healthstone!");
            ShardTracker_UpdatePartyHealthstoneList(thePartyMember, false, true);
        end
    else
        ShardTrackerPrint("ShardTracker_DebugHealthstonePartyCode: Couldn't find "..thePartyMember.." in the party!");
    end
end


-----------------------------------------------------------------------------------
--
-- Nifty little function to view any lua object as text
--
-----------------------------------------------------------------------------------
function asText(obj)

    visitRef = {}
    visitRef.n = 0

    asTxRecur = function(obj, asIndex)
        if type(obj) == "table" then
            if visitRef[obj] then
                return "@"..visitRef[obj]
            end
            visitRef.n = visitRef.n +1
            visitRef[obj] = visitRef.n

            local begBrac, endBrac
            if asIndex then
                begBrac, endBrac = "[{", "}]"
            else
                begBrac, endBrac = "{", "}"
            end
            local t = begBrac
            local k, v = nil, nil
            repeat
                k, v = next(obj, k)
                if k ~= nil then
                    if t > begBrac then
                        t = t..", "
                    end
                    t = t..asTxRecur(k, 1).."="..asTxRecur(v)
                end
            until k == nil
            return t..endBrac
        else
            if asIndex then
                -- we're on the left side of an "="
                if type(obj) == "string" then
                    return obj
                else
                    return "["..obj.."]"
                end
            else
                -- we're on the right side of an "="
                if type(obj) == "string" then
                    return '"'..obj..'"'
                else
                    return tostring(obj)
                end
            end
        end
    end -- asTxRecur

    return asTxRecur(obj)
end -- asText



--[[  
                                INSTRUCTIONS FOR MODIFYING PARTYFRAME.XML
                                
      NOTE:  This is only applicable IF you don't use the PartyFrame.xml file included with
      ShardTracker.  You'll also need UnitPopup.xml, which comes with ShardTracker or is
      installed with Cosmos.

      Step 1.
      
      Change to your WoW root directory (usually \Program Files\World of Warcraft\).
      
      Step 2.
      
      Change to the Interface\FrameXML\ directory.
      
      Step 3.
      
      Find the file PartyFrame.xml and open it in any text editor (wordpad, notepad, etc).  (If this
      file isn't in the FrameXML directory, you'll need to get the pre-edited one that comes with 
      ShardTracker.  Try downloading ShardTracker from a location that has the full version including
      the PartyFrame.XML file.)
      
      Step 4.
      
      Search for the exact phrase:
      
      parentHealthBar" 
      
      including the ending doublequotes.
      
      Step 5.
      
      You should see the lines of code below, in addition to all the surrounding code:
            
      --------------------------------------------------------------------------------------------
      
                    </Frame>
                </Frames>
            </Frame>
            <StatusBar name="$parentHealthBar" inherits="TextStatusBar">

      --------------------------------------------------------------------------------------------

      You're going to insert the code here, as indicated by "----> INSERT HERE <----":
      
      --------------------------------------------------------------------------------------------

                    </Frame>
      ----> INSERT HERE <----
                </Frames>
            </Frame>
            <StatusBar name="$parentHealthBar" inherits="TextStatusBar">
      
      --------------------------------------------------------------------------------------------

      Step 6.
      
      Cope the XML code below, starting with AND INCLUDING <!-- ShardTracker button START -->
      and ending with AND INCLUDING <!-- ShardTracker button END -->. 
      
      Step 7.
      
      Paste this code into the indicated location in PartyFrame.XML.  It'll end up looking like this:

      --------------------------------------------------------------------------------------------

                    </Frame>
<!-- ShardTracker button START -->
                    <Button name="$parentShardTrackerIcon" hidden="false" enableMouse="true" toplevel="true">
                        <Size>
                            <AbsDimension x="25" y="25"/>
                        </Size>
                        <Anchors>
                            <Anchor point="BOTTOMLEFT">
                                <Offset>
                                    <AbsDimension x="25" y="35"/>
                                </Offset>
                            </Anchor>
                        </Anchors>
                        <NormalTexture file="Interface\AddOns\ShardTracker\Images\ShardHealth"/>
                        <Scripts>
                            <OnEnter>
                                ShardTracker_ShowShardButtonTooltip(SHARDTRACKER_PARTYHEALTH_TIP1, SHARDTRACKER_PARTYHEALTH_TIP2, SHARDTRACKER_PARTYHEALTH_TIP3);
                            </OnEnter>
                            <OnLeave>
                                GameTooltip:Hide();
                            </OnLeave>          
                        </Scripts>
                    </Button>
<!-- ShardTracker button END -->
                </Frames>
            </Frame>
            <StatusBar name="$parentHealthBar" inherits="TextStatusBar">

      --------------------------------------------------------------------------------------------

    Only insert the code starting at <!-- ShardTracker button START --> and ending
    at <!-- ShardTracker button END -->.  The surrounded code is included here only for reference.
    
    Step 8.
    
    Now save PartyFrame.xml and you're good to go.
    
]]--